"use strict";

(() => {

	//<editor-fold desc="Scope Definitions">

	let adminBar = document.getElementById('admin-bar'),
		searchBar = adminBar.querySelector('.admin-bar-search'),
		searchInput = searchBar.querySelector('input'),
		adminMenu = document.getElementById('admin-bar-menu'),
		noResults = adminBar.querySelector('.admin-bar-search-noresults'),
		performSearch, filterResults, queueResultsSearch, resetResults,
		closeBar, openBar,
		resultsSearchTimer = null;

	//</editor-fold>



	//<editor-fold desc="Search Functions">

	performSearch = query => {
		if(query) {
			searchBar.classList.add('has-search');
			adminMenu.classList.add('has-search');
			queueResultsSearch(query);
		}
		else {
			searchInput.value = '';
			searchBar.classList.remove('has-search');
			adminMenu.classList.remove('has-search');
			resetResults();
		}
	};

	queueResultsSearch = query => {
		if(resultsSearchTimer !== null) {
			// There is currently a timer in place, clear that out.
			clearTimeout(resultsSearchTimer);
		}

		resultsSearchTimer = setTimeout(() => { filterResults(query); }, 200);
	};

	resetResults = () => {
		if(resultsSearchTimer !== null) {
			// There is currently a timer in place, clear that out,
			// we'll take care of rendering the results here.
			clearTimeout(resultsSearchTimer);
		}

		noResults.style.display = 'none';

		adminMenu.querySelectorAll('.has-sub').forEach(el => {

			let children = el.querySelectorAll('li');

			el.style.display = 'block';
			el.classList.remove('active');

			children.forEach(subel => {
				subel.style.display = 'block';
			});
		});
	};

	filterResults = query => {
		let hasMatches = false;
		//console.log('Filtering results for ', query);
		noResults.style.display = 'none';

		// We're performing a case insensitive search, so ensure the incoming query is lowercase too.
		query = query.toLowerCase();

		adminMenu.querySelectorAll('.has-sub').forEach(el => {
			let label = el.querySelector('.top-level-label'),
				children = el.querySelectorAll('li');

			if(label.innerHTML.toLowerCase().match(query) !== null) {
				// The entire label matches the query, display all children.
				children.forEach(subel => {
					subel.classList.add('search-match');
				});

				el.classList.add('search-match', 'active');
				hasMatches = true;
			}
			else {
				// Label doesn't match, but check the children.
				let matches = 0;

				children.forEach(subel => {
					if(subel.querySelector('a').title.toLowerCase().match(query) !== null) {
						subel.classList.add('search-match');
						matches += 1;
						hasMatches = true;

						if(subel.classList.contains('child-page')) {
							// This is a sub page; show the parent too.
							subel.closest('.children-menu').closest('li').classList.add('search-match');
						}
					}
					else {
						// Title does not match; but make sure there are no children that do
						if(!subel.querySelectorAll('.search-match').length) {
							subel.classList.remove('search-match');
						}
					}
				});

				if(matches > 0) {
					el.classList.add('search-match', 'active');
				}
				else {
					el.classList.remove('search-match', 'active');
				}
			}
		});

		adminMenu.querySelectorAll('li.tile').forEach(el => {
			let link = el.querySelector('a');

			if(link.title.toLowerCase().match(query) !== null) {
				el.classList.add('search-match', 'active');
				hasMatches = true;
			}
			else {
				el.classList.remove('search-match', 'active');
			}
		});

		if(!hasMatches) {
			noResults.style.display = 'block';
		}
	};

	//</editor-fold>

	//<editor-fold desc="Bar UX Functions">

	openBar = () => {
		// We use Cookies here to allow the server to be aware of the change.
		// This allows the server to set necessary classes before the body is loaded to prevent UI jumping.
		Cookies.remove('ADMINBAR');

		document.body.classList.remove('adminbar-closed');
		document.body.classList.add('adminbar-open');
		Cookies.set('ADMINBAR', 'open');
		adminBar.classList.add('active');
		setTimeout(() => { window.dispatchEvent(new Event('resize')); }, 350);
	};

	closeBar = () => {
		// We use Cookies here to allow the server to be aware of the change.
		// This allows the server to set necessary classes before the body is loaded to prevent UI jumping.
		Cookies.remove('ADMINBAR');

		document.body.classList.add('adminbar-closed');
		document.body.classList.remove('adminbar-open');
		Cookies.set('ADMINBAR', 'closed');
		adminBar.classList.remove('active');
		setTimeout(() => { window.dispatchEvent(new Event('resize')); }, 350);
	};

	//</editor-fold>


	//<editor-fold desc="Event listeners on common shared resources">

	searchInput.addEventListener('keyup', evt => {
		if(evt.key === 'Escape') {
			performSearch('');
		}
		else {
			performSearch(evt.target.value);
		}
	});

	//</editor-fold>


	//<editor-fold desc="Event listeners on simple non-recurring objects">

	adminBar.querySelectorAll('.top-level-label').forEach(el => {
		el.addEventListener('click', evt => {
			evt.target.closest('li').classList.toggle('active');
			evt.preventDefault();
		});
	});

	// For users clicking on the arrow and not the label
	adminBar.querySelectorAll('.expand-hint').forEach(el => {
		el.addEventListener('click', evt => {
			evt.target.closest('li').classList.toggle('active');
			evt.preventDefault();
		});
	});

	adminBar.querySelectorAll('.set-lang-target').forEach(el => {
		el.addEventListener('click', evt => {
			Cookies.remove('LANG');
			Cookies.set('LANG', evt.target.dataset.lang);
			Core.Reload;
			evt.preventDefault();
		});
	});

	adminBar.querySelector('.admin-bar-clear-search').addEventListener('click', evt => {
		performSearch('');
		evt.preventDefault();
	});

	adminBar.querySelector('.toggle-admin-bar').addEventListener('click', evt => {
		if(document.body.classList.contains('adminbar-closed')) {
			openBar();
		}
		else {
			closeBar();
		}

		evt.preventDefault();
	});

	adminBar.querySelector('.admin-bar-search-all a').addEventListener('click', evt => {
		window.location.href = Core.ROOT_URL + 'page/search?q=' + encodeURI(searchInput.value);
		evt.preventDefault();
	});

	adminBar.querySelector('.toggle-admin-bar-search').addEventListener('click', evt => {
		openBar();
		searchInput.focus();
		evt.preventDefault();
	});

	/**
	 * Add an event listener on the document itself to watch for focus/unfocus events.
	 *
	 * This generally does not have an effect other than on Mobile,
	 * though it can be used to style the interface a little.
	 */
	document.addEventListener('click', evt => {
		if(!evt.target.closest('.toggle-admin-bar')) {
			// Only check if it was NOT on the toggle portion, this handles its own logic separately.
			if(evt.target.closest('.admin-bar')) {
				adminBar.classList.add('active');
			}
			else {
				adminBar.classList.remove('active');
			}
		}
	});

	//</editor-fold>

	// Default state for admin bar
	if(!document.body.classList.contains('adminbar-closed')) {
		document.body.classList.add('adminbar-open')
	}

	// If the admin bar is open and more than 50% of the page width, hide it.
	// This addresses the issue of a user on mobile clicking a link in the bar,
	// redirecting to the new page, and having the bar still visible,
	// while having the added benefit
	// of a better UX because the user can see the bar slide
	// out of the way automatically when the new page is available.
	if(
		!document.body.classList.contains('adminbar-closed') &&
		document.body.offsetWidth <= 900
		//(adminBar.offsetWidth * 2 >= document.body.offsetWidth)
	) {
		setTimeout(() => { closeBar(); }, 350);
	}
})();