# Cron System

Default cron system for Core Plus, provides a number of time-based hooks
that allow jobs to be executed automatically.

## Slots Available

* 1-minute
* 5-minute
* 15-minute
* 30-minute
* hourly
* 2-hour
* 3-hour
* 6-hour
* 12-hour
* daily
* weekly
* monthly

## Scheduling

Most time slots will run automatically as soon as the appropriate time delta between the last
run has elapsed.  The exceptions are daily, weekly, and monthly.
(Though by default they will run at the first available time.)

Configuring the cron schedule can be done through `Configure` -> `Cron Jobs`.

### Daily Scheduling

The daily slot can be scheduled to select the hour to run.
This is beneficial if you want daily jobs to always run at 6am for example.

**Daylight Savings Time**

Due to DST, daily cron jobs that have a specific hour set will allow a wider amount of variance
than usual.  This is changed from the default 5 seconds to 1 hour.

This is done to allow jobs that are scheduled at 6a server time to always run at 6a,
(even if only 23 hours have elapsed technically).

If a daily schedule is left as default to run any time, this exception does not apply,
and the default 23h 59m 55s delta applies.

### Weekly Scheduling

The weekly slot can be scheduled to select the day of the week to run.

### Monthly Scheduling

The monthly slot can be scheduled to select the day of the month to run.
For simplicity, this is limited to the 1st through the 28th.


## Hooking Into Crons

To hook into the cron system for an automated job, simply add the requested hook in your component.xml.

    <hook name="/cron/[name-of-cron-timeslot]" call="SomeClass::HookCron"/>
    
This public static function must return either `true` or `false`, 
(depending on failure or success), and echo any message to be recorded in the log.  


## Handling Output / Logging

All output from jobs ran as a cron is automatically redirected to the log
to be reviewable by system administrators.  This means simple `echo` or `CLI::Print*`
commands are effective in your jobs.

If no output is provided, "Job completed without output" is simply listed in the log.


## Handling Errors

All exceptions should be handled internally in your job,
and boolean `false` can be returned to indicate failure.

The error output can be printed directly to stdout, which will get redirected to the log. 