{css}<style>
	.collapsible {
		max-height: 50em;
        transform: scaleY(1);
		transform-origin: top;
		transition: max-height 0.5s, transform 0.5s;
    }

	.collapsible.collapsed {
		transform: scaleY(0);
		max-height: 0;
		overflow: hidden;
		padding: 0;
    }
</style>{/css}

<main class="width-wrapper">
	<h1>
        {t 'STRING_CRON_CONFIGURE'}
	</h1>

	<p class="message-tutorial">
		Crons are automated jobs that run on your site to do background tasks such as
		cleanup logs, check for updates, send emails, and other various tasks.<br/><br/>
		In order for automated jobs to run the script must be called periodically,
		<a href="#cron-setup-instructions">see Setup Instructions below</a> to set this up.
	</p>

	<label>
		Enable/Disable Automatic Crons<br/>
		<a href="#" class="button button-toggle button-size-large enable-disable-cron" title="Toggle Cron System">
			<span></span>
		</a>
	</label>
	<br/>


    {$form->render()}

	<hr/>

	{if $is_enabled}
		<div id="cron-results">

		</div>
		<hr/>
	{else}
		<p class="message-warning">
			Automatic cron jobs are set to be disabled and will not run.
		</p>
	{/if}

	<div class="collapsible collapsed cron-expander">
		<h2 id="cron-setup-instructions">Setup Instructions</h2>

		{if $is_cpanel}
			<p>
				Log into your CPanel management dashboard (your provider can assist with this), scroll down to "Advanced",
				and click on "Cron Jobs".  Alternatively, you can search for "Cron" with the inline search feature
				at the top of the page.
			</p>
            {img src="assets/images/tutorials/cron/cron-configuration-cpanel-1.png"}
			<br/><br/>

			<p>
				On the Cron page, scroll down to setup a new cron job.  Ensure "Common Settings" are set to "Once Per Minute (* * * * *).
			</p>
            {img src="assets/images/tutorials/cron/cron-configuration-cpanel-2.png"}
			<br/><br/>

			<p>
				Once set, enter the following command and click "Add New Cron Job".
			</p>

			<pre><code>curl -s {$url}cron -o /dev/null
</code></pre>
		{else}
			<p>
				Run <code>crontab -e</code> and enter the following line in the text editor:
			</p>

			<pre><code># Execute the hourly webcron for {$sitename}
*  *  *  *  *  curl -s {$url}cron -o /dev/null
</code></pre>

			<p>
				Alternatively, <code>wget -q {$url}cron</code> does the trick too if you prefer wget over cURL.
			</p>
		{/if}
	</div>


</main>





{script location="foot"}<script>
	let enableButton = document.querySelector('.enable-cron'),
		disableButton = document.querySelector('.disable-cron'),
		enableDisableButton = document.querySelector('.enable-disable-cron'),
		expander = document.querySelector('.cron-expander'),
		statusResults = document.getElementById('cron-results');

	enable = () => {
		expander.classList.remove('collapsed');
		enableDisableButton.classList.add('active');
		document.querySelector('[name="config[/cron/enabled]"][value=true]').checked = 'checked';

		document.getElementById('formradioinput-config-cron-enabled').style.display = 'none';
		document.querySelector('.formselectinput-config-cron-run-daily').style.display = 'block';
		document.querySelector('.formselectinput-config-cron-run-weekly').style.display = 'block';
		document.querySelector('.formselectinput-config-cron-run-monthly').style.display = 'block';

		enableDisableButton.title = 'Crons enabled - click to disable';
	}

	disable = () => {
		expander.classList.add('collapsed');
		enableDisableButton.classList.remove('active');
		document.querySelector('[name="config[/cron/enabled]"][value=false]').checked = 'checked';

		document.getElementById('formradioinput-config-cron-enabled').style.display = 'none';
		document.querySelector('.formselectinput-config-cron-run-daily').style.display = 'none';
		document.querySelector('.formselectinput-config-cron-run-weekly').style.display = 'none';
		document.querySelector('.formselectinput-config-cron-run-monthly').style.display = 'none';

		enableDisableButton.title = 'Crons disabled - click to enable';
	}

	toggle = () => {
		if(enableDisableButton.classList.contains('active')) {
			disable();
		}
		else {
			enable();
		}
	}

	loadStatus = () => {
		statusResults.innerHTML = '<p><i class="icon icon-spin icon-spinner"></i> Loading Status...</p>';

		Core.Ajax(
			Core.ROOT_URL + 'cron/status',
			{
				callback: response => {
					setTimeout(loadStatus, 60000);

					if(response.system_status === 'disabled') {
						statusResults.innerHTML = '<p class="message-warning">Crons are disabled</p>';
					}
					else if(response.system_status === 'stale') {
						statusResults.innerHTML = '<p class="message-warning">Crons do not appear to be running, last run - ' + response.last_run_formatted + '</p>' +
							'<p>If you just enabled crons, give them a minute to start.</p>';
					}
					else {
						let html = '<p class="message-success">Crons are running!</p>';

						html += '<ul class="listing-collection">';

						Object.keys(response.crons).forEach(key => {
							html += '<li class="collection-item">' +
								'<div class="collection-item-title">' +
								key +
								' - last run ' +
								response.crons[key].run_formatted +
								'</div>';

							if(response.crons[key].details) {
								html += '<div class="collection-item-description">' + response.crons[key].details + '</div>';
							}

							html += '<div class="collection-item-controls">' +
								'<a href="' + response.crons[key].link + '" title="View Results Log">' +
								'<i class="icon icon-eye"></i>' +
								'<span>View Log</span>' +
								'</a>';

							html += '</li>';
						});

						html += '</ul>';
						statusResults.innerHTML = html;
					}
				}
			}
		)
	}

	// Set the initial status.
	if(document.querySelector('[name="config[/cron/enabled]"][checked]').value === 'true') {
		enable();
	}
	else {
		disable();
	}

	enableDisableButton.addEventListener('click', evt => {
		toggle();
		evt.preventDefault();
	});

	if(statusResults) {
		loadStatus();
	}
</script>{/script}