var FormBuilder = new function(){
	var $group, $cfs, self = this, newCounter = 0, initialized = false, fieldTypes = {},
		fieldBasicForms = {},
		fieldAdvancedForms = {}, buildOptionHTML;

	buildOptionHTML = function(data, basic){
		var i, html = '';

		if(basic){
			html += '<div class="' + data.type + '-' + data.name + '">';
		}
		else{
			html += '<div class="' + data.type + '-' + data.name + ' formelement form' + data.type + '">';
		}

		if(!basic && typeof data.label !== 'undefined'){
			// Advanced forms get the label prepended at the top.
			html += data.label;
		}

		if(data.type == 'textarea' || data.type == 'text'){
			html += data.type == 'text' ? '<input type="text"' : '<textarea';
			for(i in data){
				if(i != 'type' && i != 'title' && i != 'description'){
					html += ' ' + i + '="' + data[i] + '"';
				}
			}
			html += data.type == 'text' ? '/>' : '></textarea>';
		}
		else if(data.type == 'select'){
			html += '<select name="' + data.name + '"';
			for(i in data){
				if(i != 'type' && i != 'title' && i != 'description' && i != 'options'){
					html += ' ' + i + '="' + data[i] + '"';
				}
			}
			html += '>'; // End the select
			// And begin the options.
			for(i in data.options){
				html += '<option value="' + i + '">' + data.options[i] + '</option>';
			}
			html += '</select>';
		}

		if(!basic && typeof data.description !== 'undefined'){
			html += '<p class="form-element-description">' + data.description + '</p>';
		}

		html += '</div>'; // Close the div

		return html;
	};

	/**
	 * Initialize the formbuilder interface.
	 */
	this.init = function(opts){
		var genericForms = {};

		if(initialized){
			// Only run this logic once.
			return;
		}

		initialized = true;
		
		// Allow options to be passed in to control how this system behaves.
		if(typeof opts === 'undefined'){
			// Initialize options as a blank object.
			opts = {};
		}

		if(typeof opts.dest === 'undefined'){
			opts.dest = '#core-forms-tabsgroup-tstring_basic';
		}
		$group = $(opts.dest);

		// Allow the type of fields to be dynamically created and set completely from Javascript.
		if(typeof opts.fields === 'undefined'){
			// Standard fields.
			fieldTypes = {
				captcha: "Captcha",
				checkboxes: "Checkboxes",
				country: "Country Select",
				date: "Date",
				datetime: "Date & Time",
				email: "Email Address",
				file: "File Upload",
				radio: "Radio Buttons",
				state: "State Select",
				select: "Select",
				submit: "Submit Button",
				text: "Text Field",
				textarea: "Textarea",
				time: "Time"
			};
		}
		else{
			fieldTypes = opts.fields;
		}

		// Basic form values for the various supported types.
		fieldBasicForms.textarea = [
			{
				type: 'text',
				name: 'cols',
				size: '5',
				placeholder: 'Cols',
				title: 'Number of columns wide'
			},
			{
				type: 'text',
				name: 'rows',
				size: '5',
				placeholder: 'Rows',
				title: 'Number of rows tall'
			}
		];
		fieldBasicForms.checkboxes = [
			{
				type: 'textarea',
				name: 'options',
				placeholder: 'Options, one per line.'
			}
		];
		fieldBasicForms.radio = [
			{
				type: 'textarea',
				name: 'options',
				placeholder: 'Options, one per line.'
			}
		];
		fieldBasicForms.select = [
			{
				type: 'textarea',
				name: 'options',
				placeholder: 'Options, one per line.'
			}
		];
		fieldBasicForms.submit = [
			{
				type: 'text',
				name: 'value',
				placeholder: 'Button Value',
				title: 'Text of the submit button'
			}
		];

		fieldBasicForms['*'] = [];


		if(typeof opts.wildcardBasicFields !== 'undefined') {
			// Simple option
			fieldBasicForms['*'] = opts.wildcardBasicFields;
		}
		else if(typeof opts.basicFields !== 'undefined') {
			// Advanced option, various field options can be defined
			Object.keys(opts.basicFields).forEach(key => {
				if(key === 'wildcard') {
					// Allow this to be aliased
					fieldBasicForms[ '*' ] = opts.basicFields[ key ];
				}
				else {
					fieldBasicForms[ key ] = opts.basicFields[ key ];
				}
			});
		}

		// Some generic forms that are shared throughout!
		genericForms.description = {
			label: 'Description',
			name: 'description',
			type: 'textarea',
			description: 'Additional help or description below the form field to be shown to the user.'
		};
		genericForms.placeholder = {
			label: 'Placeholder Text',
			name: 'placeholder',
			placeholder: 'Placeholder Text',
			type: 'text',
			description: 'Default placeholder text before the user enters text.'
		};
		genericForms.required = {
			label: 'Required',
			name: 'required',
			type: 'select',
			options: {No: 'Not Required', Yes: 'Required'},
			description: 'Set to required to require the user to input a value.'
		};
		genericForms.readonly = {
			label: 'Readonly',
			name: 'readonly',
			type: 'select',
			options: {No: 'Not Readonly (Editable)', Yes: 'Readonly!'},
			description: 'Set to readonly to prevent user changing of this field.'
		};
		genericForms.secure = {
			label: 'Secure',
			name: 'secure',
			type: 'select',
			options: { '': 'Standard Field', encrypted: 'Encrypted Field (Stored in DB)', 'private': 'Private Field (NOT Stored in DB)' },
			description: '* Standard fields are saved in plain text and sent in plain text.<br/> \
		       * Encrypted fields are stored in the database as an encrypted value, (but still accessable).<br/> \
		       * Private fields are NOT STORED at all and are only sent over encrypted email during the original submission.  You must have GPG working in order to use this option.'
		};

		fieldAdvancedForms.text = [
			genericForms.description, genericForms.placeholder, genericForms.required, genericForms.readonly, genericForms.secure
		];
		fieldAdvancedForms.textarea = [
			genericForms.description, genericForms.placeholder, genericForms.required, genericForms.secure
		];
		fieldAdvancedForms.radio = [
			genericForms.description, genericForms.required
		];
		fieldAdvancedForms.file = [
			genericForms.description, genericForms.required
		];
		fieldAdvancedForms.email = [
			genericForms.description, genericForms.required
		];
		fieldAdvancedForms.select = [
			genericForms.description
		];
		fieldAdvancedForms.select = [
			genericForms.description, genericForms.required, genericForms.secure
		];

		// Set the necessary markup
		$group.append("<div class='custom-fields'></div>" + '<button id="add-field-button"><i class="icon icon-add"></i> Add Field</button>');

		$cfs = $group.find('.custom-fields');

		/**********************************************
		 *         Bind the necessary events
		 *********************************************/

		$('#add-field-button').click(function(e){
			self.addField();
			return false;
		});

		// populate and display the form preview
		$('#preview-form-button').click(function(e){
			formRenderPreview();

			$('#form-preview').dialog('open');
			return false;
		});

		// Click to show more information
		$group.on('click', '.more-information-link', function(){
			$(this).closest('.custom-field').toggleClass('expanded');
			return false;
		});
		// Click to show less information
		$group.on('click', '.less-information-link', function(){
			$(this).closest('.custom-field').toggleClass('expanded');
			return false;
		});

		// Click to delete CF
		$group.on('click', '.delete-link', function(){
			$(this).closest('.custom-field').remove();
			return false;
		});

		$group.on('mouseover', '.custom-field', function(){
			$(this).closest('.custom-field').addClass('hover');
		});

		$group.on('mouseout', '.custom-field', function(){
			$(this).closest('.custom-field').removeClass('hover');
		});

		$group.on('change', '.field-type-input', function(){
			var $this = $(this),
				v = $this.val(),
				basicOptionHTML = '', advancedOptionHTML = '',
				$cf = $this.closest('.custom-field'),
				id = $cf.data('id'),
				$beo = $cf.find('.basic-information-extra-options'),
				$aeo = $cf.find('.advanced-information-extra-options'),
				options = $cf.data('options'), i;

			// .basic-information-extra-options

			// Generate the basic options for this input type.
			if(typeof fieldBasicForms[v] !== 'undefined'){
				for(i in fieldBasicForms[v]){
					basicOptionHTML += buildOptionHTML(fieldBasicForms[v][i], true);
				}
			}

			// Add the wildcard for all basic options.
			// This allows the external script to add form elements to display on all elements.
			for(i in fieldBasicForms['*']){
				basicOptionHTML += buildOptionHTML(fieldBasicForms['*'][i], true);
			}


			// Generate the advanced options for this input type.
			if(typeof fieldAdvancedForms[v] !== 'undefined'){
				for(i in fieldAdvancedForms[v]) {
					advancedOptionHTML += buildOptionHTML(fieldAdvancedForms[v][i], false);
				}
			}
			else{
				advancedOptionHTML = '<p>This field type does not support any advanced options.</p>';
			}

			$beo.html(basicOptionHTML);

			// Each field contained herein needs the ID prefixed so that the server knows what data is what.
			$beo.find(':input').each(function(){
				var $this = $(this),
					n = $this.attr('name');

				$this.attr('name', 'fields[' + id + '][options][' + n + ']');

				if(options !== null && typeof options[n] != 'undefined'){
					$this.val(options[n]).change();
				}
			});

			$aeo.html(advancedOptionHTML);

			// Each field contained herein needs the ID prefixed so that the server knows what data is what.
			$aeo.find(':input').each(function(){
				var $this = $(this),
					n = $this.attr('name');

				$this.attr('name', 'fields[' + id + '][options][' + n + ']');

				if(options !== null && typeof options[n] != 'undefined'){
					$this.val(options[n]).change();
				}
			});
		});


		// Initialize the sortables
		$cfs.sortable({
			placeholder: 'custom-field-droppable',
			handle: '.drag-handle'
		});
	};

	/**
	 * Add a new field to the interface.
	 *
	 * The one parameter must be either empty or an object with at least the following keys:
	 * id:                  null|string
	 * formid:              null|string
	 * type:                string
	 * title:               string
	 * conditional:         0|1
	 * conditional_element: string
	 * conditional_value:   string
	 * options:             null|object
	 *
	 * @param data
	 */
	this.addField = function (data){

		// Ensure that the system is initialized, (can be called multiple times)
		this.init();

		var fieldHTML,
			$newfield,
			id, i;

		if(typeof data == 'undefined'){
			data = {
				id: null,
				formid: null,
				type: 'text',
				title: '',
				conditional: 0,
				conditional_element: '',
				conditional_value: '',
				options: null
			};
		}

		// Set the ID as either the variable pulled from the options or newly generated.
		id = data.id ? data.id : 'new-' + ++newCounter;

		// Generate the HTML for this field
		fieldHTML = '<div class="custom-field clearfix">';
		// Add the drag option
		fieldHTML += '<div class="drag-handle" title="Drag Custom Field"><i class="icon icon-sort"></i></div>';
		// Add the type selection input
		fieldHTML += '<div class="basic-information">';
		fieldHTML += '<div class="formelement formselect formrequired formselect-fieldtype"><select class="field-type-input" name="type" required="required">';
		for(i in fieldTypes){
			fieldHTML += '<option value="' + i + '">' + fieldTypes[i] + '</option>';
		}
		// Close the selection div
		fieldHTML += '</select></div>';
		// Every form element has a name/Title!
		fieldHTML += '<div class="formelement formtextinput formrequired formerror formtextinput-title"> \
			<input maxlength="" name="title" type="text" value="" placeholder="Title"> \
			</div>';

		// Add the basic options
		fieldHTML += '<div class="basic-information-extra-options"></div>';
		// And the controls for this form element
		fieldHTML += '<div class="custom-field-right-links">' +
			'<a href="#" class="delete-link" title="Delete Custom Field"><i class="icon icon-trash"></i></a>' +
			'</div>';

		// And the more/less information links
		fieldHTML += '<div style="clear:both;">' +
			'<a class="more-information-link" href="#"><i class="icon icon-angle-double-down"></i> More Details</a>' +
			'<a class="less-information-link" href="#"><i class="icon icon-angle-double-up"></i> Less Details</a>' +
			'</div>';
		// Close the basic information DIV
		fieldHTML += '</div>';

		// And start the advanced placeholder, hidden by default.
		fieldHTML += '<div class="advanced-information-extra-options"></div>';

		// Lastly, close the form element div
		fieldHTML += '</div>';

		$newfield = $(fieldHTML);

		$cfs.append($newfield);

		$newfield.data('id', id);
		$newfield.data('options', data.options);

		// Each field contained herein needs the ID prefixed so that the server knows what data is what.
		$newfield.find(':input').each(function(){
			var $this = $(this),
				n = $this.attr('name');

			$this.attr('name', 'fields[' + id + '][' + n + ']');

			if(typeof data[n] != 'undefined'){
				$this.val(data[n]).change();
			}
		});
	};

	/**
	 * Add demo fields to the form, useful for new forms that should just have name, email, comments.
	 */
	this.addDemoFields = function (){
		this.addField({
			id: null,
			type: 'text',
			title: 'Name',
			options: null
		});

		this.addField({
			id: null,
			type: 'email',
			title: 'Email',
			options: null
		});

		this.addField({
			id: null,
			type: 'textarea',
			title: 'Comments',
			options: null
		});

		this.addField({
			id: null,
			type: 'submit',
			title: '',
			options: {
				value: 'Contact Us'
			}
		});
	}
};