<?php
/**
 * Class file for the model FormBuilderFieldModel
 *
 * @package FormBuilder
 * @author Nicholas Hinsch <nicholas@evalagency.com>
 */

use Core\Model;


class FormBuilderFieldModel extends Model {

	public static $ExportGroup = 'Custom Forms';

	/**
	 * Schema definition for FormBuilderFieldModel
	 * @todo Fill this in with your model structure
	 *
	 * @static
	 * @var array
	 */
	public static $Schema = array(
		'id' => [
			'type' => Model::ATT_TYPE_UUID,
		],
		'formid' => array(
			'type'     => Model::ATT_TYPE_UUID_FK,
			'required' => true,
			'null'     => false,
			'formtype' => 'system',
		),
		'type' => array(
			'type' => Model::ATT_TYPE_STRING,
			'required' => true,
			'maxlength' => 64,
			'form' => array(
				'title' => 'Field Type',
				'description' => 'The type of field to display on the custom form',
				'group' => 'Form Elements',
				'grouptype' => 'tabs',
			)
		),
		'title' => array(
			'type' => Model::ATT_TYPE_STRING,
			'null' => false,
			'required' => true,
			'form' => array(
				'group' => 'Form Elements',
				'grouptype' => 'tabs',
			),
		),
		'conditional' => array(
			'type' => Model::ATT_TYPE_BOOL,
			'required' => true,
			'form' => array(
				'group' => 'Form Elements',
				'grouptype' => 'tabs',
			),

		),
		'conditional_element' => array(
			'type' => Model::ATT_TYPE_STRING,
			'form' => array(
				'group' => 'Form Elements',
				'grouptype' => 'tabs',
			),

		),
		'conditional_value' => array(
			'type' => Model::ATT_TYPE_STRING,
			'form' => array(
				'group' => 'Form Elements',
				'grouptype' => 'tabs',
			),

		),
		'options' => array(
			'type' => Model::ATT_TYPE_DATA,
			'encoding' => Model::ATT_ENCODING_JSON,
		    'comment' => 'JSON encoded set of data',
		),
		'weight' => array(
			'type' => Model::ATT_TYPE_INT,
		    'null' => false,
		    'default' => 0,
		),
		'created'   => array(
			'type' => Model::ATT_TYPE_CREATED,
			'null' => false,
		),
		'updated'   => array(
			'type' => Model::ATT_TYPE_UPDATED,
			'null' => false,
		),
	);

	/**
	 * Index definition for FormBuilderFieldModel
	 *
	 * @static
	 * @var array
	 */
	public static $Indexes = array(
		'primary' => array('id'),
	);

	/**
	 * Get the field options of this field as an array
	 *
	 * @return array
	 */
	public function getOptions(){
		$o = $this->get('options');
		
		// Options may be null, in that case simply return a blank array.
		return $o ? $o : [];
	}

	/**
	 * Set the field options of this field from an array
	 *
	 * @param $opts array
	 */
	public function setOptions($opts){
		$this->set('options', $opts);
	}
}
