<?php
/**
 * File for class GeoAddressController definition in the coreplus project
 *
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20131104.1541
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\Controller;
use Core\View;
use GeoLookup\Lookup;


/**
 * A short teaser of what GeoAddressController does.
 *
 * More lengthy description of what GeoAddressController does and why it's fantastic.
 *
 * <h3>Usage Examples</h3>
 *
 *
 * @todo Write documentation for GeoAddressController
 * <h4>Example 1</h4>
 * <p>Description 1</p>
 * <code>
 * // Some code for example 1
 * $a = $b;
 * </code>
 *
 *
 * <h4>Example 2</h4>
 * <p>Description 2</p>
 * <code>
 * // Some code for example 2
 * $b = $a;
 * </code>
 *
 *
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class GeoAddressController extends Controller {

	/**
	 * View to get the provinces for a given country
	 */
	public function getprovinces() {
		$view             = $this->getView();
		$request          = $this->getRequest();
		$selected_country = $request->getParameter(0) ? $request->getParameter(0) : REMOTE_COUNTRY;
		
		$provincejs = [];
		$provinces = Lookup::GetProvinces($selected_country);
		foreach($provinces as $p) {
			$provincejs[] = [
				'country' => $p->country,
				'code'    => $p->province,
				'name'    => $p->getProvinceName()
			];
		}

		$view->mode = View::MODE_AJAX;
		$view->contenttype = View::CTYPE_JSON;
		$view->jsondata = $provincejs;
	}

	/**
	 * Set the timezone for a user's session
	 *
	 * Completely public because it's set from Javascript.
	 * This is important because we want the user's local settings.
	 *
	 * Asking the user for their timezone when it's already set on their local workstation
	 * is redundant and a hassle.
	 *
	 * @core-request-method POST
	 */
	public function settimezone(string $timezone) {
		$view = $this->getView();

		$view->mode = View::MODE_NOOUTPUT;
		$view->record = false;

		\Core\Session::Set('/geocode/timezone', $timezone);
	}
}
