<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace GeoLookup;

use Core\Router;
use Core\Session;
use Exception;

/**
 * Description of GeoHook
 *
 * @author charlie
 */
class GeoHook {
	/**
	 * Hook to load the REMOTE_* constants based on the user's IP.
	 */
	public static function CoreStarted() {
		$timezone = null;

		try{
			$result = Lookup::ByIP(REMOTE_IP);
			define('REMOTE_CONTINENT', $result->continent);
			define('REMOTE_COUNTRY', $result->country);
			define('REMOTE_PROVINCE', $result->province);
			define('REMOTE_CITY', $result->city);
			define('REMOTE_POSTAL', $result->postal);
			define('REMOTE_LAT', $result->lat);
			define('REMOTE_LNG', $result->lng);

			// Will provide a default if the timezone is not set otherwise from the automatic lookup
			$timezone = $result->timezone;
			
			$ret = true;
		}
		catch (Exception $ex) {
			$ret = false;
		}

		// Prefer the user's set timezone, this will be more accurate than a geolookup guess.
		if(Session::IsKeySet('/geocode/timezone')) {
			$timezone = Session::Get('/geocode/timezone');
		}

		define('REMOTE_TIMEZONE', $timezone);

		return $ret;
	}

	public static function PageRendering() {
		if(!\Core\user()->checkExists()) {
			// Only run this for registered users.
			// This is to prevent Session data from being generated for anonymous viewers.
			return true;
		}

		if(Session::IsKeySet('/geocode/timezone')) {
			// Skip this check if the timezone is already set.
			return true;
		}

		$script = <<<EOS
<script>
Core.Ajax(Core.ROOT_URL + 'geoaddress/settimezone', { method: 'POST', parameters: { timezone: Intl.DateTimeFormat().resolvedOptions().timeZone } });
</script>
EOS;

		$view = Router::$DefaultView;
		if($view) {
			$view->addScript($script, 'foot');
		}
	}
}
