<?php


namespace MediaManager;


use Core\Router;
use Core\Templates\Template;
use Core\View;
use Core\ViewControl;

class MediaManagerHooks {
	public static function PageControlLinks(\PageModel $page) {
		$ret = [];

		if(\Core\user()->checkAccess('g:admin') && $page->last_template) {
			// Resolve the template to ensure it exists
			try {
				$template = Template::Factory($page->last_template);
				$filename = $template->getFilename();
				// Filename will be fully resolved
				$filename = substr($filename, strlen(ROOT_PDIR));
				// This file could be in the component, theme, or custom.
				// either way we don't care; we just need the part after /pages/.
				// the navigator will be able to resolve it correctly.
				$filename = substr($filename, strpos($filename, '/pages/') + 7);

				$ret[] = [
					'icon' => 'file-code',
					'title' => 'Edit Page Template',
					'link' => '/mediamanagernavigator/file/editor?type=page&path=' . $filename,
					'group' => ViewControl::GROUP_ADMIN,
				];
			}
			catch(\Exception $e) {
				// You just don't get this option.
				\Core\log_exception($e);
			}
		}

		if(\Core\user()->checkAccess('g:admin') && $page->baseurl) {
			$customPath = 'js/page' . $page->baseurl . '.js';

			$ret[] = [
				'icon' => 'file-code',
				'title' => 'Edit Page Javascript',
				'link' => '/mediamanagernavigator/file/editor?type=assets&path=' . $customPath,
				'group' => ViewControl::GROUP_ADMIN,
			];

			$customPath = 'css/page' . $page->baseurl . '.css';

			$ret[] = [
				'icon' => 'file-image',
				'title' => 'Edit Page CSS',
				'link' => '/mediamanagernavigator/file/editor?type=assets&path=' . $customPath,
				'group' => ViewControl::GROUP_ADMIN,
			];
		}

		return $ret;
	}

	public static function AttachJS(View $view) : bool {
		$request = Router::$DefaultRequest;
		$pageModel = Router::$DefaultPageModel;

		if(\Core\Session::Get('mediamanager-inline-styleeditor')) {
			\Core\Core::LoadScriptLibrary('notify');
			$view->addScript('assets/js/mediamanager/getmatchedcssrules.js', 'head');
			$view->addScript('assets/js/mediamanager/styleeditor.js', 'foot');
			$view->addStylesheet('assets/css/mediamanager/styleeditor.css');
		}

		if($pageModel && $pageModel->baseurl) {
			// Allow support for custom javascript on specific pages.
			// This provides the user with the ability to easily do custom logic for specific pages.
			$jsCheckPath = ROOT_PDIR . 'themes/custom/assets/js/page' . $pageModel->baseurl . '.js';

			if(file_exists($jsCheckPath)) {
				$view->addScript('<script data-source="admin-page-custom">' . file_get_contents($jsCheckPath) . '</script>', 'foot');
			}


			$cssCheckPath = ROOT_PDIR . 'themes/custom/assets/css/page' . $pageModel->baseurl . '.css';

			if(file_exists($cssCheckPath)) {
				$view->addStyle('<style data-source="admin-page-custom">' . file_get_contents($cssCheckPath) . '</style>');
			}
		}

		return true;
	}
}