{css src="assets/css/mediamanager/file_editor_block.css"}{/css}
{script library="jqueryui"}{/script}
{script library="notify"}{/script}

<div class="width-wrapper">
	<h1>Template Block Editor</h1>


	<div class="block-editor-interface {if !count($panels)}no-layout-selected{else}layout-selected{/if}">
		<h2 class="block-editor-layouts-header">Available Layouts</h2>

		<p class="message-tutorial block-editor-layouts-help">
			The block template editor allows you to select what type of elements will be present
			to the content editor when creating an modifying pages that use this template.<br/><br/>
			<strong>You will not be entering the actual content here!</strong><br/><br/>
			The first step will be to select what type of overall layout to use.
		</p>

		<ul class="block-layouts tile-container" id="block-editor-layouts">

		</ul>

		<h2 class="block-editor-panels-header">Layout Block Regions</h2>

		<p class="message-tutorial block-editor-panels-help">
			Below is a preview of the structure of the selected layout.
			Blocks can be installed into different panels and rearranged as necessary.
			Installed blocks can also be moved between panels, or removed altogether.<br/><br/>
			For advanced users, the raw HTML template can be edited for each block.
		</p>

		<div class="block-editor">
			<div class="block-editor-panel-workspace">

			</div>
		</div>

		{$form->render()}

		<div class="block-editor-panels-actions">
			<div class="button-group">
				<a href="#" class="button back-script">
					<i class="icon icon-backward"></i>
					<span>Back to Template Selection</span>
				</a>
				<a href="#" class="button save-script">
					<i class="icon icon-check"></i>
					<span>Save Template</span>
				</a>
			</div>
		</div>
	</div>
</div>


<div id="block-editor-available-blocks" class="block-editor-available-blocks">
	<div class="block-panel-title">Available Blocks</div>

	<p class="message-tutorial">
		Select the block to install into the selected panel section.  Clicking on the block will install it.
	</p>

	<div class="block-editor-panel-blocks tile-container" data-block-panel="__available_blocks_list">

	</div>

</div>

{script location="foot"}<script>
	(() => {
		let panelID, panelBlocks, sortables,
			selectedPanel = null,
			installedKeys = [],
			blockSelectionModal = new Core.ModalDialog(),
			editorInterface = document.querySelector('.block-editor-interface'),
			editorForm = document.getElementById('block-editor-form'),
			availableLayouts = document.getElementById('block-editor-layouts'),
			availableBlocks = document.querySelector('[data-block-panel=__available_blocks_list]'),
			editorWorkspace = document.querySelector('.block-editor-panel-workspace');


		// Setup the modal initially
        blockSelectionModal.setFrame(document.getElementById('block-editor-available-blocks'));

		function clearPanels() {
			installedKeys = [];
			editorWorkspace.innerHTML = '';
		}

		function createLayout(layoutData) {
			let layout,
				layoutPanels = [],
				panelCount = 0,
				i;

			for(i in layoutData.panels) {
				if(layoutData.panels[i].key && layoutData.panels[i].name) {
					panelCount++;
					layoutPanels.push({
						key: layoutData.panels[i].key,
						name: layoutData.panels[i].name
					});
				}
			}

			layout = document.createElement('li');
			layout.key = layoutData.key;
			layout.templateContent = layoutData.templateContent;

			let html = layoutData.templateContent;
            {literal}
				// Strip out smarty content.
				html = html.replace(/\{[^\}]*\}/g, '');
            {/literal}

			layout.panels = layoutPanels;
			layout.innerHTML = '<div class="tile-title">' + layoutData.name + '</div><div class="tile-preview">' + html + '</div>';
			layout.classList.add('tile', 'selectable');

			availableLayouts.appendChild(layout);
		}

		function createBlock(blockData) {
			let block, innerHTML, target;

			block = document.createElement('div');
			block.classList.add('block', 'tile', 'selectable');
			innerHTML = '<span class="tile-title">' + blockData.name + '</span>';
			block.name = blockData.name;
			block.templateContent = blockData.templateContent;
			block.single = blockData.single;
			block.key = blockData.key;
			block.image = blockData.image;

			if(block.image) {
				innerHTML += '<div class="tile-image"><img src="' + block.image + '" title="' + blockData.name + '"/></div>';
			}

			innerHTML += '<a href="#" class="edit-block-link" title="Edit HTML Code"><i class="icon icon-code"></i></a>' +
				'<a href="#" class="delete-block-link" title="Remove Block"><i class="icon icon-times"></i></a>';

			if(blockData.panel) {
				target = editorWorkspace.querySelector('[data-block-panel="' + blockData.panel + '"]');

				if(blockData.key) {
					installedKeys.push(blockData.key);
				}
			}
			else {
				target = availableBlocks;
			}

			block.innerHTML = innerHTML;

			target.appendChild(block);
		}

		function installBlock(blockData) {
			if(blockData.key) {
				if(blockData.single === '1' || blockData.single === 1) {
					// This block can only be on the page once, make sure it's not already loaded.
					if(installedKeys.indexOf(blockData.key) !== -1) {
						$.notify('This element can only be loaded once on the page!', 'error');
						return;
					}
				}
				else {
					// This block has a key associated with it, ensure it's unique so multiple of these entries can be added.
					// We'll do a simple increment counter to allow data to be preserved between template updates.
					// Though obviously if block A is moved from one panel to another that data will be mvoed too,
					// but if the admin does this, then we can only assume that's what they want.
					let counter = 1;
					let newKey = blockData.key.toLowerCase(),
						rg = new RegExp(blockData.key, 'g');

					while(installedKeys.indexOf(newKey + counter) !== -1) {
					    counter++;
                    }

                    newKey = newKey + counter;

					// Swap the content of this key with the newly generated key
					blockData.templateContent = blockData.templateContent.replace(rg, newKey);
					blockData.key = newKey;
				}
			}

			createBlock(blockData);
			$.notify('Installed block ' + blockData.name + ' into ' + blockData.panel, 'success');
		}

		function rebuildSortables() {
			if(sortables) {
				// Cleanup first.
				sortables.sortable('destroy');
			}

			sortables = $('.block-editor-panel-workspace [data-block-panel]');
			sortables.addClass('tile-container');
			sortables.sortable({
				items: '.block',
				start: function(event, ui) {
					item = ui.item;
					newList = oldList = ui.item.closest('[data-block-panel]');
				},
				stop: function(event, ui) {
					let toAvailable = newList.data('block-panel') === '__available_blocks_list';

					if(oldList === newList) {
						$.notify('Respositioned block ' + item.text(), 'success');
					}
					else if(toAvailable) {
						if(item[0].key && installedKeys.indexOf(item[0].key) !== -1) {
							// Remove this from the list of registered keys
							installedKeys.splice(installedKeys.indexOf(item[0].key), 1);
						}
						item.remove();
						$.notify('Removed block ' + item.text() + ' from page template', 'success');
					}
					else {
						$.notify('Moved block ' + item.text() + ' to ' + newList.data('block-panel'), 'success');
					}
				},
				change: function(event, ui) {
					if(ui.sender) {
						newList = ui.placeholder.closest('[data-block-panel]');
					}
				},
				connectWith: '[data-block-panel]'
			}).disableSelection();
		}

        {foreach $layouts as $layout}
			createLayout({$layout|json_encode});
			{if $layout.current}
				let html = {$layout|json_encode}.templateContent;
				{literal}
					// Strip out smarty content.
					html = html.replace(/\{[^\}]*\}/g, '');
				{/literal}
				editorWorkspace.innerHTML = html;

		        // Add controls for each panel in this layout
		        editorWorkspace.querySelectorAll('[data-block-panel]').forEach(el => {
		            el.innerHTML = '<a href="#" class="panel-add-block button"><i class="icon icon-add"></i></span>Add Block</span></a>';

		            el.addEventListener('click', evt => {
		                if(evt.target.closest('.panel-add-block')) {
		                    evt.preventDefault();

		                    selectedPanel = evt.target.closest('[data-block-panel]').dataset.blockPanel;
		                    blockSelectionModal.open();
		                }
		            });
		        });
			{/if}
        {/foreach}

        {foreach $blocks as $block}
			createBlock({$block|json_encode});
        {/foreach}

		rebuildSortables();


		availableLayouts.querySelectorAll('.tile').forEach(el => {
			el.addEventListener('click', evt => {
				evt.preventDefault();

				let tile = evt.target.closest('.tile'),
					i,
					html,
					editorWorkspace = document.querySelector('.block-editor-panel-workspace');

				clearPanels();
				document.getElementById('formtextareainput-layout').value = tile.templateContent;

				html = tile.templateContent;
                {literal}
					// Strip out smarty content.
					html = html.replace(/\{[^\}]*\}/g, '');
                {/literal}
				editorWorkspace.innerHTML = html;

				// Add controls for each panel in this layout
				editorWorkspace.querySelectorAll('[data-block-panel]').forEach(el => {
                    el.innerHTML = '<a href="#" class="panel-add-block button"><i class="icon icon-add"></i></span>Add Block</span></a>';

                    el.addEventListener('click', evt => {
                        if(evt.target.closest('.panel-add-block')) {
                            evt.preventDefault();

                            selectedPanel = evt.target.closest('[data-block-panel]').dataset.blockPanel;
                            blockSelectionModal.open();
                        }
                    });
                });

				editorInterface.classList.remove('no-layout-selected');
				editorInterface.classList.add('layout-selected');
				rebuildSortables();
			});
		});

        availableBlocks.addEventListener('click', evt => {
            let block = evt.target.closest('.block');

            if(block) {
                evt.preventDefault();

                installBlock({
                    name: block.name,
                    templateContent: block.templateContent,
                    panel: selectedPanel,
                    key: block.key,
                    single: block.single,
                    image: block.image
                });

                selectedPanel = null;
                blockSelectionModal.close();
            }
        });


		document.querySelector('.back-script').addEventListener('click', evt => {
			evt.preventDefault();
			editorInterface.classList.add('no-layout-selected');
			editorInterface.classList.remove('layout-selected');
		});

		document.querySelector('.save-script').addEventListener('click', evt => {
			evt.preventDefault();

			let blockCounts = 0;

			if(!document.getElementById('formtextareainput-layout').value) {
				alert('You must select a layout first.');
				return;
			}

			editorWorkspace.querySelectorAll('[data-block-panel]').forEach(panel => {
				let panelID = panel.dataset.blockPanel, html = '', input;

				panel.querySelectorAll('.block').forEach(block => {
					blockCounts++;
					input = document.createElement('textarea');
					input.name = 'panels[' + panelID + '][]';
					input.value = block.templateContent;
					input.style.display = 'none';
					editorForm.appendChild(input);

					// HTML is now used for testing.
					html += block.templateContent + "\n\n";
				});

				//console.log(panelID); console.log(html);
			});

			if(!blockCounts) {
				alert('No blocks have been added, please add something first.');
				return;
			}

			editorForm.submit();

		});

		document.querySelector('.block-editor').addEventListener('click', evt => {
			if(evt.target.closest('.edit-block-link')) {
				let dialog = new Core.ModalDialog(),
					block = evt.target.closest('.block'),
					blockEditorTextarea;
				dialog.open();

				dialog.frame.innerHTML = '<textarea id="block-editor-block-edit" rows="10"></textarea><a href="#" class="button" id="block-editor-block-save"><span>Save Block</span></a>';

				blockEditorTextarea = document.getElementById('block-editor-block-edit');
				blockEditorTextarea.value = block.templateContent;

				document.getElementById('block-editor-block-save').addEventListener('click', evt => {
					block.templateContent = blockEditorTextarea.value;
					dialog.close();
					$.notify('Updated block content', 'success');
					evt.preventDefault();
				});

				evt.preventDefault();
			}

            if(evt.target.closest('.delete-block-link')) {
                let block = evt.target.closest('.block'),
	                blockName = block.name;

                if(block.key && installedKeys.indexOf(block.key) !== -1) {
                    // Remove this from the list of registered keys
                    installedKeys.splice(installedKeys.indexOf(block.key), 1);
                }
                block.parentElement.removeChild(block);
                $.notify('Removed block ' + blockName + ' from page template', 'success');

                evt.preventDefault();
            }

		});
	})();

</script>{/script}