<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core;
use DOMElement;
use WidgetInstanceModel;
use WidgetModel;

/**
 * Description of ComponentWidgetDefinition
 *
 * @author charlie
 */
class ComponentWidgetDefinition {
	
	public $baseurl;
	public $installable;
	public $title;
	public $component;
	
	public function __construct(DOMElement $node, string $component){
		$this->component   = $component;
		$this->baseurl     = $node->getAttribute('baseurl');
		$this->title       = $node->getAttribute('title');
		$this->installable = $node->getAttribute('installable');
	}
	
	/**
	 * Install this defined widget into the system and return true if changes were detected.
	 * 
	 * @return bool
	 */
	public function install() : bool {
		$model = new WidgetModel(-1, $this->baseurl);

		$isNew = $model->checkIsNew();
		
		if($isNew){
			// Some keys only get set on new objects so as to not overwrite user settings.
			$model->setFromArray([
				'title'      => $this->title,
			]);
		}
		
		// Standard overwrites that apply to both existing and new pages.
		$model->setFromArray([
			'installable' => $this->installable,
		]);

		$updated = $model->save();

		if($isNew) {
			// Install new admin widgets into the dashboard
			// @todo Add support for this process across different types of widgets
			if($this->installable === '/admin') {
				$weight = WidgetInstanceModel::Count(
					[
						'widgetarea' => 'Admin Dashboard',
						'page_baseurl' => '/admin',
					]
				) + 1;

				$wi = new WidgetInstanceModel();
				$wi->setFromArray(
					[
						'baseurl'      => $this->baseurl,
						'page_baseurl' => '/admin',
						'widgetarea'   => 'Admin Dashboard',
						'weight'       => $weight
					]
				);
				$wi->save();
			}
		}
		
		return $updated;
	}
	
	/**
	 * Uninstall this defined widget from the system
	 * 
	 * @return bool
	 */
	public function uninstall() : bool {
		// These are always global widgets.
		$model = new WidgetModel(-1, $this->baseurl);
		return ($model->delete());
	}
}
