<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Filestore\Contents;

use Core\Filestore\ContentsInterface;
use Core\Filestore\File;

/**
 * Description of ContentPHP
 *
 * @author charlie
 */
class ContentPHP implements ContentsInterface {
	private $_file;
	private $_classes;
	private $_uses;
	private $_interfaces;
	private $_traits;
	private $_lastNamespace;

	public function __construct(File $file) {
		$this->_file = $file;
	}
	
	/**
	 * 
	 * @return array Hash containing name, extends, and implements
	 */
	public function getClasses() : array {
		if($this->_classes === null){
			$this->_inspect();
		}

		return $this->_classes;
	}
	
	/**
	 * 
	 * @return array Hash containing name
	 */
	public function getInterfaces(){
		if($this->_interfaces === null){
			$this->_inspect();
		}
		return $this->_interfaces;
	}
	
	/**
	 * 
	 * @return array Hash containing name
	 */
	public function getTraits(){
		if($this->_traits === null){
			$this->_inspect();
		}
		return $this->_traits;
	}
	
	private function _inspect(){
		$this->_classes = [];
		$this->_uses = [];
		$this->_interfaces = [];
		$this->_traits = [];
		$this->_lastNamespace = null;
		
		$contents = file_get_contents($this->_file->getFilename());			
		$tokens = token_get_all($contents);
		$last = '';
		$typeHint = null;
		$lastSaved = null;
					
		foreach($tokens as $idx => $t){
			if(is_array($t)){
				if(
					$t[0] === T_NAMESPACE ||
					$t[0] === T_CLASS ||
					$t[0] === T_USE ||
					$t[0] === T_INTERFACE ||
					$t[0] === T_TRAIT
				){
					// These are all line-level codes that need to set the type of command the token is currently on.
					$typeHint = $t[0];
					$last = '';
					unset($lastSaved);
					$lastSaved = null;
				}
				elseif($t[0] === T_EXTENDS){
					// This changes the current typehint and saves the last!
					if($typeHint === T_CLASS){
						$lastSaved =& $this->_inspectSetClassName($last);
					}
					elseif($typeHint === T_INTERFACE){
						$lastSaved =& $this->_inspectSetInterfaceName($last);
					}
					$typeHint = $t[0];
					$last = '';
				}
				elseif($t[0] === T_IMPLEMENTS){
					// Class implements something!  This can follow a class declaration or an extend declaration.
					if($typeHint === T_CLASS){
						$lastSaved =& $this->_inspectSetClassName($last);
					}
					elseif($typeHint === T_EXTENDS){
						$this->_inspectSetNExtends($last, $lastSaved);
					}
					$typeHint = $t[0];
					$last = '';
				}
				elseif( $t[0] === T_NS_SEPARATOR ){
					// NS seperators append the last updated key
					$last .= '\\';
				}
				elseif($t[0] === T_STRING && $typeHint !== null){
					// This is some string and has a typehint!
					$last .= $t[1];
				}
			}
			else{
				// NULL codes indicate an end of line execution and should clear out the current line hint and save the contents.
				if($typeHint === T_NAMESPACE){
					$this->_lastNamespace = $last;
				}
				elseif($typeHint === T_CLASS){
					if($last) {
						$this->_inspectSetClassName($last);
					}
				}
				elseif($typeHint === T_EXTENDS){
					if($last) {
						$this->_inspectSetNExtends($last, $lastSaved);
					}
				}
				elseif($typeHint === T_USE){
					if($last) {
						$this->_uses[] = $last;
					}
				}
				elseif($typeHint === T_INTERFACE){
					if($last) {
						$this->_inspectSetInterfaceName($last);
					}
				}
				elseif($typeHint === T_TRAIT){
					if($last) {
						$this->_inspectSetTraitName($last);
					}
				}
				elseif($typeHint === T_IMPLEMENTS){
					if($last) {
						$this->_inspectSetNImplements($last, $lastSaved);
					}
				}
				$typeHint = null;
				$last = '';
			}
		}
	}
	
	private function &_inspectSetInterfaceName($value){
		$this->_interfaces[] = [
			'name' => ($this->_lastNamespace ? $this->_lastNamespace . '\\' : '') . $value,
			'extends' => null,
		];
		
		return $this->_interfaces[ count($this->_interfaces) - 1 ];
	}
	
	private function _inspectSetTraitName($value){
		$this->_traits[] = [
			'name' => ($this->_lastNamespace ? $this->_lastNamespace . '\\' : '') . $value,
		];
	}
	
	private function &_inspectSetClassName($value){
		$this->_classes[] = [
			'name'       => ($this->_lastNamespace ? $this->_lastNamespace . '\\' : '') . $value,
			'extends'    => null,
			'implements' => [],
		];

		return $this->_classes[ sizeof($this->_classes) - 1 ];
	}
	
	private function _inspectSetNExtends($value, &$object){
		// Iterate over the list of currently set uses and resolve this extend if it's set.
		$check = '\\' . (($pos = strpos($value, '\\')) !== false ? substr($value, 0, $pos) : $value);
		$aliased = false;
		
		foreach($this->_uses as $use){
			if(strrpos($use, $check) === strlen($use) - strlen($check)){
				// This extend is aliased to a longer namespace!
				$value = $use . substr($value, strlen($check) - 1);
				$aliased = true;
				break;
			}
			if($use === $value){
				// The use is top-level and contained in its entirety in the use declaration.
				$aliased = true;
				break;
			}
		}
		
		if(!$aliased && $this->_lastNamespace){
			// "Implements" does not have a registered namespace, so prefix it!
			$value = $this->_lastNamespace . '\\' . $value;
		}
		
		$object['extends'] = $value;
	}
	
	private function _inspectSetNImplements($value, &$object){
		// Iterate over the list of currently set uses and resolve this extend if it's set.
		$check = '\\' . (($pos = strpos($value, '\\')) !== false ? substr($value, 0, $pos) : $value);
		$aliased = false;
		
		foreach($this->_uses as $use){
			if(strrpos($use, $check) === strlen($use) - strlen($check)){
				// This extend is aliased to a longer namespace!
				$value = $use . substr($value, strlen($check) - 1);
				$aliased = true;
				break;
			}
			if($use === $value){
				// The use is top-level and contained in its entirety in the use declaration.
				$aliased = true;
				break;
			}
		}
		
		if(!$aliased && $this->_lastNamespace){
			// "Implements" does not have a registered namespace, so prefix it!
			$value = $this->_lastNamespace . '\\' . $value;
		}
		
		$object['implements'][] = $value;
	}
}
