<?php


namespace Core\Formatter;

use Core\Converter\MDtoHTML\MarkdownToHTMLConverter;


class MarkdownFormatter {
	/**
	 * Format markdown content with the default formatter.
	 *
	 * This should ONLY be used with markdown populated by authenticated / authorized users,
	 * as the rendered HTML is not stripped and would pose a security risk if used by untrusted sources.
	 * 
	 * @param string $markdown Raw markdown content
	 * @param string|null $format
	 *
	 * @return string Rendered HTML from the markdown input (when HTML is requested)
	 */
	public static function DefaultFormat(string $markdown, string $format = null) : string {
		if($format === 'text/html') {
			// Only format this content if the HTML formatter was actually requested.
			return MarkdownToHTMLConverter::DefaultConvert($markdown);
		}
		else {
			// Other formats just get the raw markdown.
			return $markdown;
		}
	}
}