<?php
/**
 * File for class Loader definition in the coreplus project
 * 
 * @package Core\I18N
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20140326.2321
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\I18N;
use Core\Cache;
use Core\Component;
use Core\ConfigHandler;
use Core\Core;
use Core\Router;
use Core\Utilities\Profiler\Profiler;
use Core\Utilities\Profiler\ProfilerEvent;


/**
 * A short teaser of what Loader does.
 *
 * More lengthy description of what Loader does and why it's fantastic.
 *
 * <h3>Usage Examples</h3>
 *
 *
 * @todo Write documentation for Loader
 * <h4>Example 1</h4>
 * <p>Description 1</p>
 * <code>
 * // Some code for example 1
 * $a = $b;
 * </code>
 *
 *
 * <h4>Example 2</h4>
 * <p>Description 2</p>
 * <code>
 * // Some code for example 2
 * $b = $a;
 * </code>
 *
 * 
 * @package Core\i18n
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class I18NLoader {
	protected static $Strings;

	protected static $IsLoaded = false;

	protected static $DefaultLanguage = null;
	protected static $UserLanguage = null;
	protected static $FallbackLanguage = 'en';
	
	protected static $_StringLookupCache = [];

	protected static $_JSStrings = [];
	/**
	 * decimal_point 	Decimal point character
	 * thousands_sep 	Thousands separator
	 * grouping 	Array containing numeric groupings
	 * int_curr_symbol 	International currency symbol (i.e. USD)
	 * currency_symbol 	Local currency symbol (i.e. $)
	 * mon_decimal_point 	Monetary decimal point character
	 * mon_thousands_sep 	Monetary thousands separator
	 * mon_grouping 	Array containing monetary groupings
	 * positive_sign 	Sign for positive values
	 * negative_sign 	Sign for negative values
	 * int_frac_digits 	International fractional digits
	 * frac_digits 	Local fractional digits
	 * p_cs_precedes 	TRUE if currency_symbol precedes a positive value, FALSE if it succeeds one
	 * p_sep_by_space 	TRUE if a space separates currency_symbol from a positive value, FALSE otherwise
	 * n_cs_precedes 	TRUE if currency_symbol precedes a negative value, FALSE if it succeeds one
	 * n_sep_by_space 	TRUE if a space separates currency_symbol from a negative value, FALSE otherwise
	 * p_sign_posn
	 *     0 - Parentheses surround the quantity and currency_symbol
	 *     1 - The sign string precedes the quantity and currency_symbol
	 *     2 - The sign string succeeds the quantity and currency_symbol
	 *     3 - The sign string immediately precedes the currency_symbol
	 *     4 - The sign string immediately succeeds the currency_symbol
	 * n_sign_posn
	 *     0 - Parentheses surround the quantity and currency_symbol
	 *     1 - The sign string precedes the quantity and currency_symbol
	 *     2 - The sign string succeeds the quantity and currency_symbol
	 *     3 - The sign string immediately precedes the currency_symbol
	 *     4 - The sign string immediately succeeds the currency_symbol
	 *
	 * @var array
	 */
	protected static $LocaleConv = [];
	
	// <editor-fold defaultstate="collapsed" desc="protected static $Languages = [ ... ]">
	protected static $Languages = [
		'af'    => [
			'lang'    => 'STRING_LANG_AFRIKAANS',
			'dialect' => 'STRING_DIALECT_AFRIKAANS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ar'    => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_ARABIC',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_AE' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_UAE',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_BH' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_BAHRAIN',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_DZ' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_ALGERIA',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_EG' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_EGYPT',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_IQ' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_IRAQ',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_JO' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_JORDAN',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_KW' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_KUWAIT',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_LB' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_LEBANON',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_LY' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_LIBYA',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_MA' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_MOROCCO',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_OM' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_OMAN',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_QA' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_QATAR',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_SA' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_SAUDI_ARABIA',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_SY' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_SYRIA',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_TN' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_TUNISIA',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'ar_YE' => [
			'lang'    => 'STRING_LANG_ARABIC',
			'dialect' => 'STRING_DIALECT_YEMEN',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'be'    => [
			'lang'    => 'STRING_LANG_BYELORUSSIAN',
			'dialect' => 'STRING_DIALECT_BYELORUSSIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'bg'    => [
			'lang'    => 'STRING_LANG_BULGARIAN',
			'dialect' => 'STRING_DIALECT_BULGARIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'bo'    => [
			'lang'    => 'STRING_LANG_TIBETAN',
			'dialect' => 'STRING_DIALECT_TIBETAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'bo_CN' => [
			'lang'    => 'STRING_LANG_TIBETAN',
			'dialect' => 'STRING_DIALECT_CHINA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'bo_IN' => [
			'lang'    => 'STRING_LANG_TIBETAN',
			'dialect' => 'STRING_DIALECT_INDIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'bs'    => [
			'lang'    => 'STRING_LANG_BOSNIAN',
			'dialect' => 'STRING_DIALECT_BOSNIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ca'    => [
			'lang'    => 'STRING_LANG_CATALAN',
			'dialect' => 'STRING_DIALECT_CATALAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'cs'    => [
			'lang'    => 'STRING_LANG_CZECH',
			'dialect' => 'STRING_DIALECT_CZECH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'da'    => [
			'lang'    => 'STRING_LANG_DANISH',
			'dialect' => 'STRING_DIALECT_DANISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de'    => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_STANDARD',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de_AT' => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_AUSTRIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de_CH' => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_SWISS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de_DE' => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_GERMANY',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de_LI' => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_LIECHTENSTEIN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'de_LU' => [
			'lang'    => 'STRING_LANG_GERMAN',
			'dialect' => 'STRING_DIALECT_LUXEMBOURG',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'el'    => [
			'lang'    => 'STRING_LANG_GREEK',
			'dialect' => 'STRING_DIALECT_GREEK',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en'    => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_ENGLISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_AU' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_AUSTRALIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_BZ' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_BELIZE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_CA' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_CANADIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_GB' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_BRITISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_IE' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_IRELAND',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_JM' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_JAMAICA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_NZ' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_NEW_ZEALAND',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_TT' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_TRINIDAD',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_US' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_UNITED_STATES',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'en_ZA' => [
			'lang'    => 'STRING_LANG_ENGLISH',
			'dialect' => 'STRING_DIALECT_SOUTH_AFRICA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es'    => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_SPAIN_TRADITIONAL',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_AR' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_ARGENTINA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_BO' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_BOLIVIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_CL' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_CHILE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_CO' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_COLOMBIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_CR' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_COSTA_RICA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_DO' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_DOMINICAN_REPUBLIC',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_EC' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_ECUADOR',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_ES' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_SPAIN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_GT' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_GUATEMALA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_HN' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_HONDURAS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_MX' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_MEXICAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_NI' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_NICARAGUA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_PA' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_PANAMA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_PE' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_PERU',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_PR' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_PUERTO_RICO',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_PY' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_PARAGUAY',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_SV' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_EL_SALVADOR',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_UY' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_URUGUAY',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'es_VE' => [
			'lang'    => 'STRING_LANG_SPANISH',
			'dialect' => 'STRING_DIALECT_VENEZUELA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'et'    => [
			'lang'    => 'STRING_LANG_ESTONIAN',
			'dialect' => 'STRING_DIALECT_ESTONIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'eu'    => [
			'lang'    => 'STRING_LANG_BASQUE',
			'dialect' => 'STRING_DIALECT_BASQUE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fa'    => [
			'lang'    => 'STRING_LANG_FARSI',
			'dialect' => 'STRING_DIALECT_FARSI',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'fi'    => [
			'lang'    => 'STRING_LANG_FINNISH',
			'dialect' => 'STRING_DIALECT_FINNISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fo'    => [
			'lang'    => 'STRING_LANG_FAEROESE',
			'dialect' => 'STRING_DIALECT_FAEROESE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr'    => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_STANDARD',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr_BE' => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_BELGIUM',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr_CA' => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_CANADIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr_CH' => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_SWISS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr_FR' => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_FRANCE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'fr_LU' => [
			'lang'    => 'STRING_LANG_FRENCH',
			'dialect' => 'STRING_DIALECT_LUXEMBOURG',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ga'    => [
			'lang'    => 'STRING_LANG_IRISH',
			'dialect' => 'STRING_DIALECT_IRISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'gd'    => [
			'lang'    => 'STRING_LANG_GAELIC',
			'dialect' => 'STRING_DIALECT_SCOTS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'gd_IE' => [
			'lang'    => 'STRING_LANG_GAELIC',
			'dialect' => 'STRING_DIALECT_IRISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'gl'    => [
			'lang'    => 'STRING_LANG_GALICIAN',
			'dialect' => 'STRING_DIALECT_GALICIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'he'    => [
			'lang'    => 'STRING_LANG_HEBREW',
			'dialect' => 'STRING_DIALECT_HEBREW',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		'hi'    => [
			'lang'    => 'STRING_LANG_HINDI',
			'dialect' => 'STRING_DIALECT_HINDI',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'hr'    => [
			'lang'    => 'STRING_LANG_CROATIAN',
			'dialect' => 'STRING_DIALECT_CROATIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'hu'    => [
			'lang'    => 'STRING_LANG_HUNGARIAN',
			'dialect' => 'STRING_DIALECT_HUNGARIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'hy'    => [
			'lang'    => 'STRING_LANG_ARMENIAN_ARMENIA',
			'dialect' => 'STRING_DIALECT_ARMENIAN_ARMENIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'id'    => [
			'lang'    => 'STRING_LANG_INDONESIAN',
			'dialect' => 'STRING_DIALECT_INDONESIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'in'    => [
			'lang'    => 'STRING_LANG_INDONESIAN',
			'dialect' => 'STRING_DIALECT_INDONESIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'is'    => [
			'lang'    => 'STRING_LANG_ICELANDIC',
			'dialect' => 'STRING_DIALECT_ICELANDIC',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'it'    => [
			'lang'    => 'STRING_LANG_ITALIAN',
			'dialect' => 'STRING_DIALECT_ITALIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'it_CH' => [
			'lang'    => 'STRING_LANG_ITALIAN',
			'dialect' => 'STRING_DIALECT_ITALIAN_SWISS',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ja'    => [
			'lang'    => 'STRING_LANG_JAPANESE',
			'dialect' => 'STRING_DIALECT_JAPANESE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ko'    => [
			'lang'    => 'STRING_LANG_KOREAN',
			'dialect' => 'STRING_DIALECT_KOREAN',
			'charset' => 'kr',
			'dir'     => 'ltr',
		],
		'ko_KP' => [
			'lang'    => 'STRING_LANG_KOREA',
			'dialect' => 'STRING_DIALECT_NORTH',
			'charset' => 'kr',
			'dir'     => 'ltr',
		],
		'ko_KR' => [
			'lang'    => 'STRING_LANG_KOREA',
			'dialect' => 'STRING_DIALECT_SOUTH',
			'charset' => 'kr',
			'dir'     => 'ltr',
		],
		'lt'    => [
			'lang'    => 'STRING_LANG_LITHUANIAN',
			'dialect' => 'STRING_DIALECT_LITHUANIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'lv'    => [
			'lang'    => 'STRING_LANG_LATVIAN',
			'dialect' => 'STRING_DIALECT_LATVIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'mk'    => [
			'lang'    => 'STRING_LANG_FYRO_MACEDONIAN',
			'dialect' => 'STRING_DIALECT_FYRO_MACEDONIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'mk_MK' => [
			'lang'    => 'STRING_LANG_MACEDONIAN',
			'dialect' => 'STRING_DIALECT_MACEDONIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ms'    => [
			'lang'    => 'STRING_LANG_MALAYSIAN',
			'dialect' => 'STRING_DIALECT_MALAYSIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'mt'    => [
			'lang'    => 'STRING_LANG_MALTESE',
			'dialect' => 'STRING_DIALECT_MALTESE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'nb'    => [
			'lang'    => 'STRING_LANG_NORWEGIAN_BOKMAL',
			'dialect' => 'STRING_DIALECT_NORWEGIAN_BOKMAL',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'nl'    => [
			'lang'    => 'STRING_LANG_DUTCH',
			'dialect' => 'STRING_DIALECT_STANDARD',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'nl_BE' => [
			'lang'    => 'STRING_LANG_DUTCH',
			'dialect' => 'STRING_DIALECT_BELGIUM',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'nn'    => [
			'lang'    => 'STRING_LANG_NORWEGIAN_NYNORSK',
			'dialect' => 'STRING_DIALECT_NORWEGIAN_NYNORSK',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'no'    => [
			'lang'    => 'STRING_LANG_NORWEGIAN',
			'dialect' => 'STRING_DIALECT_NORWEGIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'pl'    => [
			'lang'    => 'STRING_LANG_POLISH',
			'dialect' => 'STRING_DIALECT_POLISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'pt'    => [
			'lang'    => 'STRING_LANG_PORTUGUESE',
			'dialect' => 'STRING_DIALECT_PORTUGAL',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'pt_BR' => [
			'lang'    => 'STRING_LANG_PORTUGUESE',
			'dialect' => 'STRING_DIALECT_BRAZIL',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'rm'    => [
			'lang'    => 'STRING_LANG_RHAETO_ROMANIC',
			'dialect' => 'STRING_DIALECT_RHAETO_ROMANIC',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ro'    => [
			'lang'    => 'STRING_LANG_ROMANIAN',
			'dialect' => 'STRING_DIALECT_ROMANIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ro_MO' => [
			'lang'    => 'STRING_LANG_ROMANIAN',
			'dialect' => 'STRING_DIALECT_MOLDAVIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ru'    => [
			'lang'    => 'STRING_LANG_RUSSIAN',
			'dialect' => 'STRING_DIALECT_RUSSIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ru_MO' => [
			'lang'    => 'STRING_LANG_RUSSIAN',
			'dialect' => 'STRING_DIALECT_MOLDAVIA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sb'    => [
			'lang'    => 'STRING_LANG_SORBIAN',
			'dialect' => 'STRING_DIALECT_SORBIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sk'    => [
			'lang'    => 'STRING_LANG_SLOVAK',
			'dialect' => 'STRING_DIALECT_SLOVAK',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sl'    => [
			'lang'    => 'STRING_LANG_SLOVENIAN',
			'dialect' => 'STRING_DIALECT_SLOVENIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sq'    => [
			'lang'    => 'STRING_LANG_ALBANIAN',
			'dialect' => 'STRING_DIALECT_ALBANIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sr'    => [
			'lang'    => 'STRING_LANG_SERBIAN',
			'dialect' => 'STRING_DIALECT_SERBIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sv'    => [
			'lang'    => 'STRING_LANG_SWEDISH',
			'dialect' => 'STRING_DIALECT_SWEDISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sv_FI' => [
			'lang'    => 'STRING_LANG_SWEDISH',
			'dialect' => 'STRING_DIALECT_FINLAND',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sx'    => [
			'lang'    => 'STRING_LANG_SUTU',
			'dialect' => 'STRING_DIALECT_SUTU',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'sz'    => [
			'lang'    => 'STRING_LANG_SAMI',
			'dialect' => 'STRING_DIALECT_LAPPISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'th'    => [
			'lang'    => 'STRING_LANG_THAI',
			'dialect' => 'STRING_DIALECT_THAI',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'tn'    => [
			'lang'    => 'STRING_LANG_TSWANA',
			'dialect' => 'STRING_DIALECT_TSWANA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'tr'    => [
			'lang'    => 'STRING_LANG_TURKISH',
			'dialect' => 'STRING_DIALECT_TURKISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ts'    => [
			'lang'    => 'STRING_LANG_TSONGA',
			'dialect' => 'STRING_DIALECT_TSONGA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'uk'    => [
			'lang'    => 'STRING_LANG_UKRAINIAN',
			'dialect' => 'STRING_DIALECT_UKRAINIAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'ur'    => [
			'lang'    => 'STRING_LANG_URDU',
			'dialect' => 'STRING_DIALECT_URDU',
			'charset' => 'utf-8',
			'dir'     => 'rtl',
		],
		've'    => [
			'lang'    => 'STRING_LANG_VENDA',
			'dialect' => 'STRING_DIALECT_VENDA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'vi'    => [
			'lang'    => 'STRING_LANG_VIETNAMESE',
			'dialect' => 'STRING_DIALECT_VIETNAMESE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'cy'    => [
			'lang'    => 'STRING_LANG_WELSH',
			'dialect' => 'STRING_DIALECT_WELSH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'xh'    => [
			'lang'    => 'STRING_LANG_XHOSA',
			'dialect' => 'STRING_DIALECT_XHOSA',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'yi'    => [
			'lang'    => 'STRING_LANG_YIDDISH',
			'dialect' => 'STRING_DIALECT_YIDDISH',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'zh'    => [
			'lang'    => 'STRING_LANG_CHINESE',
			'dialect' => 'STRING_DIALECT_CHINESE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'zh_CN' => [
			'lang'    => 'STRING_LANG_CHINESE',
			'dialect' => 'STRING_DIALECT_PRC',
			'charset' => 'GB2312',
			'dir'     => 'ltr',
		],
		'zh_HK' => [
			'lang'    => 'STRING_LANG_CHINESE',
			'dialect' => 'STRING_DIALECT_HONG_KONG',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'zh_SG' => [
			'lang'    => 'STRING_LANG_CHINESE',
			'dialect' => 'STRING_DIALECT_SINGAPORE',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'zh_TW' => [
			'lang'    => 'STRING_LANG_CHINESE',
			'dialect' => 'STRING_DIALECT_TAIWAN',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
		'zu'    => [
			'lang'    => 'STRING_LANG_ZULU',
			'dialect' => 'STRING_DIALECT_ZULU',
			'charset' => 'utf-8',
			'dir'     => 'ltr',
		],
	];
	// </editor-fold>

	public static function Init(){

		if(self::$IsLoaded){
			return;
		}

		$profiler = new ProfilerEvent('I18N Loader Scan');
		$profiler->start();

		self::$IsLoaded = true;
		
		// Try to set the default language to the user's preference
		if(\Core\Router::$DefaultRequest !== null && ($preferred = \Core\Router::$DefaultRequest->getPreferredLocale())) {
			// User has a preferred locale, try to set that.
			if(self::SetDefaultLanguage($preferred)) {
				$profiler->addEventString('Set user-preferred locale of ' . $preferred);
			}
		}

		if(!self::$UserLanguage) {
			// Set the default language based on Core's configuration.
			$default = ConfigHandler::Get('/core/language/site_default');
			self::SetDefaultLanguage($default);
			$profiler->addEventString('Set site-default locale of ' . $default);
		}

		
		// Will keep track of the source files to pull I18N info from
		$files = [];
		// Will contain a hashable string of file modification stamps; useful to know if the cache is outdated without opening them.
		$fileCache = '';
		
		$langs = explode('|', ConfigHandler::Get('/core/language/languages_enabled'));
		$sources = [];
		foreach($langs as $l) {
			$sources[] = $l . '.yml';
			
			// Also include the base if the dialect is requested as a fallback.
			if(strpos($l, '_') !== false){
				$sources[] = substr($l, 0, strpos($l, '_')) . '.yml';
			}
		}
		
		// Remove any dupes; (en_UK and en_US will both include en.yml).
		$sources = array_unique($sources);

		$profiler->addEventString('Loading ' . count($sources) . ' locales');
		
		
		// The directories to check for those files
		$checks = [];
		foreach(Core::GetComponents() as $c){
			/** @var Component $c */
			$checks[] = $c->getBaseDir() . 'i18n/';
		}
		// Include the active theme and custom overrides
		$checks[] = ROOT_PDIR . 'themes/' . ConfigHandler::Get('/theme/selected') . '/i18n/';
		$checks[] = ROOT_PDIR . 'themes/custom/i18n/';
		
		
		// Now that $sources and $checks are compiled, merge the two!
		foreach($checks as $dir) {
			foreach($sources as $file) {
				if(file_exists($dir . $file)) {
					$files[] = $dir . $file;
					$fileCache .= filemtime($dir . $file) . ':';
				}
			}
		}

		$profiler->addEventString('Found ' . count($files) . ' locale files to be loaded');
		
		$checksum = md5($fileCache);
		$cachekey = 'core-i18n-strings';
		if(
			$checksum === Cache::Get('core-i18n-checksum') &&
			($cached = Cache::Get($cachekey))
		){
			$profiler->addEventString('Valid cache checksum located, retrieving results from there');
			$profiler->stop();
			self::$Strings = $cached;
			return;
		}

		self::$Strings = [];

		$count = 0;
		foreach($files as $f){
			// Each file contains one language (or language+dialect),
			// and is named with the lang (or lang_DIALECT).
			$lang = basename($f, '.yml');
			$defs = yaml_parse_file($f);
			
			// Ensure the language group exists in Strings.
			if(!isset(self::$Strings[ $lang ])){
				self::$Strings[ $lang ] = [];
			}
			
			foreach($defs as $key => $translated){
				if($translated !== '') {
					// Quick check to skip non-set translations, not generally used, but can help if a component is
					// overriding the translation string of something else accidently.
					++$count;
					self::$Strings[ $lang ][ $key ] = $translated;
				}
			}
		}


		Cache::Set('core-i18n-checksum', $checksum, SECONDS_ONE_MONTH);
		Cache::Set($cachekey, self::$Strings, SECONDS_ONE_MONTH);

		$profiler->addEventString('Loaded ' . $count . ' translation definitions');
		$profiler->stop();
	}

	/**
	 * Lookup a translation string with the requested language.
	 *
	 * Will return an array with the located string, (if any), and some useful metadata for the lookup.
	 *
	 * @param string $key
	 * @param string|null $lang
	 *
	 * @return array
	 */
	public static function Get($key, $lang = null){

		// Ensure that this system has been loaded
		// Will simply return if already loaded.
		self::Init();

		if($lang === null) {
			// Set to the user-preferred language.
			$lang = self::$UserLanguage;
		}
		
		if($lang === null) {
			// User doesn't have a language set or is an automated process, try the default.
			$lang = self::$DefaultLanguage;
		}
		
		if($lang === null) {
			// Default isn't set, use the fallback language.
			$lang = self::$FallbackLanguage;
		}
		

		$match    = self::_SearchForString($key, $lang);
		$default  = self::_SearchForString($key, self::$DefaultLanguage);
		$fallback = self::_SearchForString($key, self::$FallbackLanguage);

		$result = [
			'key'       => $key,
			'lang'      => $lang,
			'found'     => $match['string'] !== null,
			'match_key' => $match['locale'],
			'match_str' => $match['string'],
		    'results'   => [
			    'DEFAULT'   => $default['string'],
			    'FALLBACK'  => $fallback['string'],
		    ],
		];

		// Ensure that something is returned!
		if($result['match_str'] === null && $default['string'] !== null){
			$result['match_key'] = $default['locale'];
			$result['match_str'] = $default['string'];
		}
		elseif($result['match_str'] === null && $fallback['string'] !== null){
			$result['match_key'] = $fallback['locale'];
			$result['match_str'] = $fallback['string'];
		}

		// Load in all the versions of this string that are available in the system!
		// This may be required because the calling script may be inquiring as to what all is available.
		$locales = self::GetLocalesAvailable();
		foreach($locales as $k => $dat){
			$result['results'][$k] = self::_SearchForString($key, $k)['string'];
		}

		return $result;
	}

	public static function FormatNumber($number, $precision = 0){
		self::Init();

		$decimal = self::$LocaleConv['decimal_point'] ?? '.';
		$sep = self::$LocaleConv['thousands_sep'] ?? ',';

		return number_format((float)$number, $precision, $decimal, $sep);
	}

	public static function GetLocaleConv($option){
		self::Init();

		return isset(self::$LocaleConv[$option]) ? self::$LocaleConv[$option] : null;
	}

	/**
	 * Get the user's currently selected language/locale.
	 *
	 * @return string
	 */
	public static function GetUsersLanguage(){
		self::Init();

		return self::$UserLanguage;
	}

	/**
	 * Get the fallback language
	 *
	 * @return string
	 */
	public static function GetFallbackLanguage(){
		self::Init();

		return self::$FallbackLanguage;
	}

	/**
	 * Get just the base languages available, (without dialects), as a valid optionset.
	 *
	 * @return array
	 */
	public static function GetBaseLanguagesAsOptions(){
		$localesAvailable = self::GetLocalesEnabled();
		$ret = [];
		foreach($localesAvailable as $k => $d){
			$base = strpos($k, '_') === false ? $k : substr($k, 0, strpos($k, '_'));
			$ret[$base] = t($d['lang']);
		}
		return $ret;
	}

	/**
	 * Get the languages available on this system as form options
	 *
	 * @return array
	 */
	public static function GetLanguagesAsOptions(){
		$localesAvailable = self::GetLocalesEnabled();
		$ret = [];
		foreach($localesAvailable as $k => $d){
			$ret[$k] = t($d['lang']) . ($d['dialect'] ? ' (' . t($d['dialect']) . ')' : '');
		}
		return $ret;
	}

	/**
	 * Get an array of locales currently available on the base system.
	 *
	 * @return array
	 */
	public static function GetLocalesAvailable(){
		$cacheKey = 'core-i18n-locales';
		$cacheTime = DEVELOPMENT_MODE ? 3600 : 604800;
		$cached = Cache::Get($cacheKey, $cacheTime);
		if($cached){
			return $cached;
		}

		exec('locale -a', $output);
		$locales = [];
		foreach($output as $line){
			if($line == 'C' || $line == 'POSIX' || $line == 'C.UTF-8'){
				// Yeah, skip these!
				continue;
			}
			if(($dotpos = strpos($line, '.')) !== false){
				// Trim anything after the ".", this is the charset which we, (in theory), don't need.
				// ex: .UTF-8 or .UTF-16.
				$line = substr($line, 0, $dotpos);
			}
			
			if(isset(self::$Languages[$line])){
				$locales[$line] = self::$Languages[$line];
			}
		}

		// Cache this so I don't have to execute the command and lookup the values all over again!
		Cache::Set($cacheKey, $locales, $cacheTime);
		return $locales;
	}

	/**
	 * Get an array (locale => [lang, dialect, charset, dir]) of all locales enabled on the system,
	 * XORed from the list of currently available locales on the native system.
	 * 
	 * @return array
	 */
	public static function GetLocalesEnabled(){
		$all = self::GetLocalesAvailable();

		$enabled = ConfigHandler::Get('/core/language/languages_enabled');
		// This is expected to be a pipe-seperated list of languages/locales enabled.
		$enabled = array_map('trim', explode('|', $enabled));
		
		// Return any enabled locale as long as it's available on the system.
		// Remap them to an array to ensure that the locale description/label is returned too.
		// This comes from the original GetLocalesAvaiable method.
		$out = [];
		foreach($enabled as $v){
			if(isset($all[$v])){
				$out[$v] = $all[$v];
			}
		}
		
		return $out;
	}

	/**
	 * Method to get all strings currently loaded in the system
	 *
	 * @return array
	 */
	public static function GetAllStrings($lang = null){
		self::Init();

		if($lang === null){
			$lang = self::$UserLanguage;
		}

		$return = [];

		// I need to run through each string one-by-one because a requested locale may not have all the included strings!
		$strings = self::$Strings[self::$FallbackLanguage];
		foreach($strings as $key => $str){
			$res = self::Get($key, $lang);

			$return[] = $res;
		}

		return $return;
	}


	/**
	 * "Keyify" a given string, usually from a /config or /permission directive.
	 *
	 * This will capitalize the string, replace all "/" characters to "_", and ltrim any prepending "_".
	 *
	 * @param string $string
	 *
	 * @return string
	 */
	public static function KeyifyString($string){
		return ltrim(str_replace('/', '_', strtoupper($string)), '_');
	}
	
	public static function SetDefaultLanguage($preferred) : bool {
		
		$localesAvailable = self::GetLocalesEnabled();
		
		// If the requested language isn't available, then don't set anything
		if(!isset($localesAvailable[$preferred])){
			return false;
		}

		$preferredAlt = $preferred . '.utf-8'; // Try to allow for variations of the different charsets.
		                                       // We don't actually care too much about which charset is used.
		// With this preferred value, set PHP's preference so its internal functions have the correct language.
		$res1 = setlocale(LC_COLLATE, $preferred, $preferredAlt);
		$res2 = setlocale(LC_CTYPE, $preferred, $preferredAlt);
		$res3 = setlocale(LC_NUMERIC, $preferred, $preferredAlt);
		$res4 = setlocale(LC_TIME, $preferred, $preferredAlt);
		$res5 = setlocale(LC_MESSAGES, $preferred, $preferredAlt);

		// DEBUG var_dump($preferred, $localesAvailable, $res1, $res2, $res3, $res4, $res5); die();
		// Currency does not get set to follow the user's preference, as the site admin determines what format to save and display currencies in.

		// Remember what the user's preferred language is so that I don't have to query the systemRequest again
		self::$UserLanguage = $preferred;

		// Cache this so number_format and money_format have the data available.
		self::$LocaleConv = localeconv();
		
		return true;
	}

	public static function LoadStringToJS(string $string) {

		// Load the original string
		$strings = [ $string ];

		if(strpos($string, '_N_') !== false) {
			// Load the numeric deviation versions
			$strings[] = str_replace('_N_', '_0_', $string);
			$strings[] = str_replace('_N_', '_1_', $string);
		}

		foreach($strings as $str) {
			if(in_array($str, self::$_JSStrings)) {
				// Skip strings already loaded.
				continue;
			}

			$lookup = self::Get($str);
			if($lookup['match_str'] !== null) {
				$val = $lookup['match_str'];
			}
			elseif($lookup['results']['DEFAULT'] !== null) {
				$val = $lookup['results']['DEFAULT'] ;
			}
			elseif($lookup['results']['FALLBACK'] !== null) {
				$val = $lookup['results']['FALLBACK'] ;
			}
			else {
				$val = '[' . $lookup['lang'] . ':' . $str . ']';
			}

			self::$_JSStrings[ $str ] = $val;
		}
	}

	public static function LoadStringsToJS(array $strings) {
		foreach($strings as $string) {
			self::LoadStringToJS($string);
		}
	}

	public static function AttachJS() {
		
		$lang = self::$UserLanguage;
		$cacheKey = 'i18n-js-basestring-' . $lang;
		$cache = Cache::Get($cacheKey);

		if($cache) {
			Profiler::GetDefaultProfiler()->record('Retrieved JS strings from cache');
			$content = $cache;
		}
		else {
			Profiler::GetDefaultProfiler()->record('Cache miss, building I18N JS strings');

			$data = [
				'strings' => new \stdClass(),
				'months' => [
					'short' => [
						t('STRING_MONTH_SHORT_JAN'),
						t('STRING_MONTH_SHORT_FEB'),
						t('STRING_MONTH_SHORT_MAR'),
						t('STRING_MONTH_SHORT_APR'),
						t('STRING_MONTH_SHORT_MAY'),
						t('STRING_MONTH_SHORT_JUN'),
						t('STRING_MONTH_SHORT_JUL'),
						t('STRING_MONTH_SHORT_AUG'),
						t('STRING_MONTH_SHORT_SEP'),
						t('STRING_MONTH_SHORT_OCT'),
						t('STRING_MONTH_SHORT_NOV'),
						t('STRING_MONTH_SHORT_DEC'),
					],
					'long' => [
						t('STRING_MONTH_LONG_JAN'),
						t('STRING_MONTH_LONG_FEB'),
						t('STRING_MONTH_LONG_MAR'),
						t('STRING_MONTH_LONG_APR'),
						t('STRING_MONTH_LONG_MAY'),
						t('STRING_MONTH_LONG_JUN'),
						t('STRING_MONTH_LONG_JUL'),
						t('STRING_MONTH_LONG_AUG'),
						t('STRING_MONTH_LONG_SEP'),
						t('STRING_MONTH_LONG_OCT'),
						t('STRING_MONTH_LONG_NOV'),
						t('STRING_MONTH_LONG_DEC'),
					],
				],
				'days' => [
					'short' => [
						t('STRING_DAY_SHORT_SUN'),
						t('STRING_DAY_SHORT_MON'),
						t('STRING_DAY_SHORT_TUE'),
						t('STRING_DAY_SHORT_WED'),
						t('STRING_DAY_SHORT_THU'),
						t('STRING_DAY_SHORT_FRI'),
						t('STRING_DAY_SHORT_SAT'),
					],
					'long' => [
						t('STRING_DAY_LONG_SUN'),
						t('STRING_DAY_LONG_MON'),
						t('STRING_DAY_LONG_TUE'),
						t('STRING_DAY_LONG_WED'),
						t('STRING_DAY_LONG_THU'),
						t('STRING_DAY_LONG_FRI'),
						t('STRING_DAY_LONG_SAT'),
					]
				]
			];

			$dataJSON = json_encode($data);

			$content = 'window.i18n = window.i18n || ' . $dataJSON . ';' . "\n";

			Profiler::GetDefaultProfiler()->record('Storing results in cache');
			Cache::Set($cacheKey, $content, SECONDS_ONE_MONTH);
		}

		Profiler::GetDefaultProfiler()->record('Retrieving page-specific application I18N strings');
		foreach(self::$_JSStrings as $k => $v) {
			$content .= 'window.i18n.strings["' . $k . '"] = ' . json_encode($v) . ";\n";
		}

		// Ensure Core JS is laoded
		\Core\CoreJSLoader::AttachCoreJS();

		Router::$DefaultView->addScript('<script>' . $content . '</script>', 'head');
		return true;
	}

	/**
	 * Search for a string given a set language.
	 *
	 * Will match an exact locale key and the base language if that's not found.
	 *
	 * @param string $string The string to find
	 * @param string $lang   The language to search on
	 *
	 * @return array
	 */
	protected static function _SearchForString($string, $lang){
		
		if(!isset(self::$_StringLookupCache[$lang])){
			self::$_StringLookupCache[$lang] = [];
		}
		
		// Cached copy?
		if(isset(self::$_StringLookupCache[$lang][$string])){
			return self::$_StringLookupCache[$lang][$string];
		}
		
		if(strpos($lang, '_') !== false){
			// Determine the base language as a fallback.
			// This is to allow en_US to match anything in en if an exact match isn't available.
			// Because having duplicate entries for en_US and en_GB is just silly!
			$base = substr($lang, 0, strpos($lang, '_'));
		}
		else{
			$base = null;
		}

		if(isset(self::$Strings[$lang]) && isset(self::$Strings[$lang][$string]) && self::$Strings[$lang][$string] !== ''){
			self::$_StringLookupCache[$lang][$string] = [
				'locale' => $lang,
				'string' => self::$Strings[$lang][$string],
			]; 
		}
		elseif($base && isset(self::$Strings[$base]) && isset(self::$Strings[$base][$string]) && self::$Strings[$base][$string] !== ''){
			self::$_StringLookupCache[$lang][$string] = [
				'locale' => $base,
				'string' => self::$Strings[$base][$string],
			];
		}
		else{
			self::$_StringLookupCache[$lang][$string] = [
				'locale' => null,
			    'string' => null,
			];
		}
		
		return self::$_StringLookupCache[$lang][$string];
	}
} 
