<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Installer;
use Core\Core;

/**
 * Description of InstallHelper
 *
 * @author charlie
 */
final class InstallHelper {
	private static $_User;
	private static $_OSData;
	
	public static function GetChecks() : array {
		
		// Start with the hard-coded checks for Core
		$checks = [
			// PHP Version
			new Checks\PHPVersionCheck(),
			// Memory limit
			new Checks\MemoryLimitCheck(),

			// Various PHP extensions that are required
			new Checks\PHPExtensionCheck('bcmath'),
			new Checks\PHPExtensionCheck('curl'),
			new Checks\PHPExtensionCheck('gd'),
			new Checks\PHPExtensionCheck('hash'),
			new Checks\PHPExtensionCheck('mbstring'),

			// Sodium hashes passwords
			new Checks\PHPExtensionCheck('sodium'),

			// MySQL backend is the default database, we need this.
			new Checks\PHPExtensionCheck('mysqlnd'),

			// While not strictly required in Core, it is used by core components and is probably useful
			new Checks\PHPExtensionCheck('sqlite3'),

			// All components use XML files for their metadata
			new Checks\PHPExtensionCheck('xml'),

			// I18N strings are stored as YAML
			new Checks\PHPExtensionCheck('yaml'),

			new Checks\PHPExtensionCheck('zip'),
			new Checks\PHPExtensionCheck('zlib'),

			// OS tools that are required by various utilities
			//new Checks\NativePackageCheck('imagemagick', 'convert'),


			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'themes/custom'),
			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'logs'),
			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'gnupg'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR, 'root'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'core/', 'private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'components/', 'private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'themes/', 'private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'logs/', 'private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'config/', 'private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'gnupg/', 'private'),

			// Hard code the various public directories, assume defaults as the configuration won't be available.
			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'files/assets'),
			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'files/public'),
			new Checks\WriteableDirectoryCheck(ROOT_PDIR . 'files/private'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'files/assets', 'assets'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'files/public', 'public'),
			new Checks\DirectoryProtectionCheck(ROOT_PDIR . 'files/private', 'private'),
		];
		
		// Load any dynamic check from the registry
		// Since Core isn't loaded yet, any Core-specific check will fail,
		// so just perform the OS-level checks here.
		foreach(Core::GetDisabledComponents() as $component){
			foreach($component->getRequires() as $requirement){
				if($requirement->getType() === 'phpextension'){
					$checks[] = new Checks\PHPExtensionCheck($requirement->getName());
				}
			}
		}
		
		return $checks;
	}
	
	public static function GetDevelopmentChecks() : array {
		$checks = [
			new Checks\PHPExtensionCheck('xdebug'),
			new Checks\NativePackageCheck('sassc'),
			new Checks\NativePackageCheck('closure-compiler'),
			new Checks\NativePackageCheck('zenity'),
			new Checks\NativePackageCheck('attr'),
		];
		
		return $checks;
	}
	
	/**
	 * Get the user this command is running as
	 * 
	 * @return string
	 */
	public static function GetUser() : string {
		if(self::$_User === null){
			self::$_User = trim(shell_exec('whoami'));
		}
		
		return self::$_User;
	}
	
	public static function GetWebServerUser() : string {
		// @todo Make this dynamic!
		return 'www-data';
	}
	
	public static function GetWebServerGroup() : string {
		// @todo Make this dynamic!
		return 'www-data';
	}
	
	
	/**
	 * Get the OS Family of the server
	 * 
	 * @return string
	 */
	public static function GetOSFamily() : string {
		if(self::$_OSData === null){
			self::_LoadOSData();
		}
		
		return 
			(self::$_OSData !== false && isset(self::$_OSData['OSFAMILY'])) ?
			self::$_OSData['OSFAMILY'] :
			'';
	}
	
	public static function GetPackageInstallCommand() : string {
		if(self::$_OSData === null){
			self::_LoadOSData();
		}
		
		return 
			(self::$_OSData !== false && isset(self::$_OSData['INSTALLCMD'])) ?
			self::$_OSData['INSTALLCMD'] :
			'';
	}

	public static function CheckIsCPanel() : bool {
		return file_exists('/usr/local/cpanel/version');
	}
	
	private static function _LoadOSData(){
		self::$_OSData = json_decode(shell_exec('bash ' . ROOT_PDIR . 'core/resources/basescript.sh -d'), true);
	}
}
