<?php


namespace Core\ListingTable\Renderer;


use Core\ListingTable\Column;
use Core\ListingTable\Table;
use Core\ListingTable\TableInterface;
use Core\Model;
use Core\Templates\Template;

class CollectionRenderer  implements RendererInterface {

	private $_page;
	private $_total;
	private $_limit;
	private $_records;
	private $_hasFilters;
	private $_filtersSet;

	/**
	 * Constructor
	 *
	 * @param Column[] $columns
	 */
	public function __construct(TableInterface $table) {
		// Set the necessary information from the Table

		$this->_page    = $table->getPage();
		$this->_total   = $table->getTotalCount();
		$this->_limit   = $table->getLimit();
		$this->_records = $table->getRecords();
		$this->_hasFilters  = $table->checkHasFilters();
		$this->_filtersSet  = $table->checkFiltersSet();
	}

	/**
	 * Render the contents of this interface with all the requested data
	 *
	 * @return array
	 *         +pager   bool   REQUIRED true/false if the pager is enabled
	 *         +options string REQUIRED HTML output of options for this interface
	 *         +body    string REQUIRED HTML output of the body
	 *         +page    int    Current page number, required if pager is set to true
	 *         +total   int    Total number of results, required if pager is set to true
	 *         +pages   int    Total number of pages, required if pager is set to true
	 */
	public function render() : array {

		$pages = (int) ceil($this->_total / $this->_limit);
		$template = Template::Factory('includes/listingtable/collection-data');

		$template->assign('pages', $pages);
		$template->assign('page', $this->_page);
		$template->assign('total', $this->_total);
		$template->assign('limit', $this->_limit);
		$template->assign('records', $this->_records);

		return [
			'body' => $template->fetch(),
			'download' => null,
		];
	}
}