<?php

namespace Core\User;

use Core\ListingTable\Table;
use Core\FilterForm;
use Core\Date\DateTime;
use UserModel;

class UserTable extends Table {
	public function __construct() {
		$this->setModelName('UserModel');
		$this->addClassname('listing-table-users');
		parent::__construct();
	}
	
	protected function _setupFilters() {
		
		// Assign the various filters that are allowed on this table.
		$this->addFilter(
			'text',
			[
				'title' => 't:STRING_SEARCH',
				'name' => 'q',
				'link' => FilterForm::LINK_TYPE_SEARCH
			]
		);
		
		$this->addFilter(
			'select',
			[
				'title' => 't:STRING_ACTIVE',
				'name' => 'active',
				'options' => ['' => 't:STRING_OPTION_PLACEHOLDER_ALL', '0' => 't:STRING_INACTIVE', '1' => 't:STRING_ACTIVE'],
				'link' => FilterForm::LINK_TYPE_STANDARD,
			]
		);
		
		$this->addFilter(
			'select',
			[
				'title' => 't:STRING_LAST_LOGGED_IN',
				'name' => 'last_login',
				// No link here; it's handled manually instead.
				'options' => [
					'' => 't:STRING_OPTION_PLACEHOLDER_ALL',
					'1' => 't:STRING_NEVER',
					'-1 day' => t('STRING_N_DAY', 1),
					'-7 days' => t('STRING_N_DAY', 7),
					'-30 days' => t('STRING_N_DAY', 30),
					'-60 days' => t('STRING_N_DAY', 60),
					'-90 days' => t('STRING_N_DAY', 90),
					'2' => 't:STRING_MORE_THAN_THREE_MONTHS',
				]
			]
		);
	}
	
	protected function _processFilters() {
		parent::_processFilters();
		
		if(($filterLastLogin = $this->getFilterValue('last_login')) != ''){
			
			if($filterLastLogin == 1){
				$this->getModelFactory()->where('last_login = 0');
			}
			elseif($filterLastLogin == 2){
				$d = new DateTime();
				$d->modify('-3 months');
				$this->getModelFactory()->where('last_login <= ' . $d->format('U'));
				$this->getModelFactory()->where('last_login > 0');
			}
			else{
				$d = new DateTime();
				$d->modify($filterLastLogin);
				$this->getModelFactory()->where('last_login >= ' . $d->format('U'));
			}
		}
	}
	
	protected function _setupColumns() {
		$schema = UserModel::GetSchema();
		$x = 0;
		foreach($schema->definitions as $col) {
			if($col->formatter !== null) {
				// Retrieve the basic columns to display
				// A simple way of achieving this is only display columns that have formatters attached.
				$x++;
				
				if($x >= 6) {
					$this->addColumnKey($col->field)->hidden();
				}
				else {
					$this->addColumnKey($col->field);
				}
			}
		}
	}
}