<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\ViewMeta;

use Core\ConfigHandler;

/**
 * Description of Title
 *
 * @author charlie
 */
class Title extends Base {
	
	public function __toString() : string {
		return $this->getSEOTitle();
	}
	
	public function getSEOTitle() : string {

		// If there is an explicit title set, then return that without modification.
		// This happens when the SEO title has been configured for this page, and as such
		// any modifications from the admin-set title should be performed.
		if($this->_content) {
			return $this->_content;
		}
		
		// Use the dynamic page title as a fail over.  This gets processed through the SEO options.
		$title = $this->_contentKey ?? '';

		// Support I18N natively
		if(strpos($title, 't:') === 0){
			$title = t(substr($title, 2));
		}
		
		// Support templates set from the configuration, (only if it's set)
		$template = ConfigHandler::Get('/core/page/title_template');
		if($template){
			$rep = [
				//Replaced with the title of the page
				'%%title%%' => $title,
				//The site's name
				'%%sitename%%' => SITENAME,
			];
			
			$title = str_replace(array_keys($rep), array_values($rep), $template);
		}
		
		return $title;
	}
	
	public function getTitle() : string {
		if(strpos($this->_contentKey, 't:') === 0){
			return t(substr($this->_contentKey, 2));
		}
		else{
			return $this->_contentKey;
		}
	}
	
	public function setTitle(string $title){
		$this->_contentKey = $title;
	}
	
	public function fetch() : array {
		$html = [];
		
		$seo   = $this->getSEOTitle();
		$title = $this->getTitle();
		
		
		// Set the common title tag from either the SEO or standard title.
		if($seo){
			$html[] = '<title>' . htmlentities($seo) . '</title>';
		}
		elseif($title){
			$html[] = '<title>' . htmlentities($title) . '</title>';
		}
		
		return $html;
	}
}
