<?php
/**
 * Defines the schema for the Insertable table
 *
 * @package Core
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */


use Core\Core;
use Core\Model;

/**
 * Class InsertableModel
 *
 * @property string $baseurl
 * @property string $name
 * @property int    $site    The site id in multisite mode, (or -1 if global)
 * @property string $value
 */
class InsertableModel extends Model {
	public static $ExportGroup = 't:STRING_SITE_CONTENT';

	public static $Schema = [
		'site' => [
			'type' => Model::ATT_TYPE_SITE,
			'formtype' => 'system',
		],
		'baseurl' => [
			'type'      => Model::ATT_TYPE_STRING,
			'maxlength' => 96,
			'required'  => true,
			'null'      => false,
		],
		'name'    => [
			'type'      => Model::ATT_TYPE_STRING,
			'maxlength' => 96,
			'required'  => true,
			'null'      => false,
		],
		'value'   => [
			'type' => Model::ATT_TYPE_DATA,
			'encoding' => Model::ATT_ENCODING_JSON,
			'null' => false,
		],
	];

	public static $Indexes = [
		'primary' => ['site', 'baseurl', 'name'],
	];
}
